@@ -20,33 +20,59 @@
 #include "utility.h"
 #include "common/asserts.h"
 
+#include <QCoreApplication>
 #include <QDateTime>
 #include <QDir>
-#include <QUrl>
 #include <QFile>
-#include <QCoreApplication>
+#include <QSettings>
+#include <QUrl>
 
 #include <sys/stat.h>
 #include <sys/types.h>
 
 #ifdef Q_OS_WIN
-#include <windows.h>
-#include <windef.h>
-#include <winbase.h>
+#include "utility_win.h"
 #include <fcntl.h>
 #include <io.h>
 #endif
 
+namespace {
+// Regarding
+// https://en.wikipedia.org/wiki/Comparison_of_file_systems#Limits
+// the file name limit appears to be about 255 chars
+// -10 for safety
+constexpr qsizetype fileNameMaxC = 255 - 10;
+
+constexpr auto replacementCharC = QLatin1Char('_');
+}
+
 namespace OCC {
 
 Q_LOGGING_CATEGORY(lcFileSystem, "sync.filesystem", QtInfoMsg)
 
 QString FileSystem::longWinPath(const QString &inpath)
 {
-#ifdef Q_OS_WIN
-    return pathtoUNC(inpath);
-#else
+#ifndef Q_OS_WIN
     return inpath;
+#else
+    Q_ASSERT(QFileInfo(inpath).isAbsolute());
+    if (inpath.isEmpty()) {
+        return inpath;
+    }
+    const QString str = QDir::toNativeSeparators(inpath);
+    const QLatin1Char sep('\\');
+
+    // we already have a unc path
+    if (str.startsWith(sep + sep)) {
+        return str;
+    }
+    // prepend \\?\ and to support long names
+
+    if (str.at(0) == sep) {
+        // should not happen as we require the path to be absolute
+        return QStringLiteral("\\\\?") + str;
+    }
+    return QStringLiteral("\\\\?\\") + str;
 #endif
 }
 
@@ -135,16 +161,14 @@ bool FileSystem::rename(const QString &originFileName,
     bool success = false;
     QString error;
 #ifdef Q_OS_WIN
-    const QString orig = longWinPath(originFileName);
+    const QString originalFileNameLong = longWinPath(originFileName);
     const QString dest = longWinPath(destinationFileName);
     if (FileSystem::isFileLocked(dest, FileSystem::LockMode::Exclusive)) {
         error = QCoreApplication::translate("FileSystem", "Can't rename %1, the file is currently in use").arg(destinationFileName);
-    } else if (FileSystem::isFileLocked(orig, FileSystem::LockMode::Exclusive)) {
+    } else if (FileSystem::isFileLocked(originalFileNameLong, FileSystem::LockMode::Exclusive)) {
         error = QCoreApplication::translate("FileSystem", "Can't rename %1, the file is currently in use").arg(originFileName);
     } else if (isLnkFile(originFileName) || isLnkFile(destinationFileName)) {
-        success = MoveFileEx((wchar_t *)orig.utf16(),
-            (wchar_t *)dest.utf16(),
-            MOVEFILE_COPY_ALLOWED | MOVEFILE_WRITE_THROUGH);
+        success = MoveFileEx((wchar_t *)originalFileNameLong.utf16(), (wchar_t *)dest.utf16(), MOVEFILE_COPY_ALLOWED | MOVEFILE_WRITE_THROUGH);
         if (!success) {
             error = Utility::formatWinError(GetLastError());
         }
@@ -216,7 +240,7 @@ bool FileSystem::uncheckedRenameReplace(const QString &originFileName,
     }
     const BOOL ok = MoveFileEx(reinterpret_cast<const wchar_t *>(orig.utf16()),
         reinterpret_cast<const wchar_t *>(dest.utf16()),
-        MOVEFILE_REPLACE_EXISTING + MOVEFILE_COPY_ALLOWED + MOVEFILE_WRITE_THROUGH);
+        MOVEFILE_REPLACE_EXISTING | MOVEFILE_COPY_ALLOWED | MOVEFILE_WRITE_THROUGH);
     if (!ok) {
         *errorString = Utility::formatWinError(GetLastError());
         qCWarning(lcFileSystem) << "Renaming temp file to final failed: " << *errorString;
@@ -354,7 +378,22 @@ QString FileSystem::fileSystemForPath(const QString &path)
             fileSystemBuffer, fileSystemBufferSize)) {
         return QString();
     }
-    return QString::fromUtf16(reinterpret_cast<const ushort *>(fileSystemBuffer));
+    return QString::fromUtf16(reinterpret_cast<const char16_t *>(fileSystemBuffer));
+}
+
+bool FileSystem::longPathsEnabledOnWindows()
+{
+    static std::optional<bool> longPathsEnabledCached = {};
+
+    if (!longPathsEnabledCached.has_value()) {
+        // https://learn.microsoft.com/en-us/windows/win32/fileio/maximum-file-path-limitation?tabs=registry#enable-long-paths-in-windows-10-version-1607-and-later
+        QSettings fsSettings(QStringLiteral("HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Control\\FileSystem"), QSettings::NativeFormat);
+        QVariant longPathsEnabled = fsSettings.value(QStringLiteral("LongPathsEnabled"));
+        qCDebug(lcFileSystem) << "LongPathsEnabled:" << longPathsEnabled;
+        longPathsEnabledCached = longPathsEnabled.value<uint32_t>() == 1;
+    }
+
+    return longPathsEnabledCached.value();
 }
 #endif
 
@@ -455,23 +494,51 @@ bool FileSystem::moveToTrash(const QString &fileName, QString *errorString)
 }
 
 #ifdef Q_OS_WIN
-Utility::Handle FileSystem::lockFile(const QString &fileName, LockMode mode)
-{
-    const QString fName = longWinPath(fileName);
-    auto accessMode = mode == LockMode::Exclusive ? 0 : FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE;
-    // Check if file exists
-    DWORD attr = GetFileAttributesW(reinterpret_cast<const wchar_t *>(fName.utf16()));
-    if (attr != INVALID_FILE_ATTRIBUTES) {
-        // Try to open the file with as much access as possible..
-        return Utility::Handle { CreateFileW(
-            reinterpret_cast<const wchar_t *>(fName.utf16()),
-            GENERIC_READ | GENERIC_WRITE,
-            accessMode,
-            nullptr, OPEN_EXISTING,
-            FILE_ATTRIBUTE_NORMAL | FILE_FLAG_BACKUP_SEMANTICS,
-            nullptr) };
+
+namespace {
+    /**
+     * This function creates a file handle with the desired LockMode
+     */
+    Utility::Handle lockFile(const QString &fileName, FileSystem::LockMode mode)
+    {
+        const QString fName = FileSystem::longWinPath(fileName);
+        int shareMode = 0;
+        int accessMode = GENERIC_READ | GENERIC_WRITE;
+        switch (mode) {
+        case FileSystem::LockMode::Exclusive:
+            shareMode = 0;
+            break;
+        case FileSystem::LockMode::Shared:
+            shareMode = FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE;
+            break;
+        case FileSystem::LockMode::SharedRead:
+            shareMode = FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE;
+            accessMode = GENERIC_READ;
+            break;
+        }
+        // Check if file exists
+        DWORD attr = GetFileAttributesW(reinterpret_cast<const wchar_t *>(fName.utf16()));
+        if (attr != INVALID_FILE_ATTRIBUTES) {
+            // Try to open the file with as much access as possible..
+            auto out = Utility::Handle{CreateFileW(reinterpret_cast<const wchar_t *>(fName.utf16()), accessMode, shareMode, nullptr, OPEN_EXISTING,
+                FILE_ATTRIBUTE_NORMAL | FILE_FLAG_BACKUP_SEMANTICS, nullptr)};
+
+            if (out) {
+                LARGE_INTEGER start;
+                start.QuadPart = 0;
+                LARGE_INTEGER end;
+                end.QuadPart = -1;
+                if (LockFile(out.handle(), start.LowPart, start.HighPart, end.LowPart, end.HighPart)) {
+                    OC_ENFORCE(UnlockFile(out.handle(), start.LowPart, start.HighPart, end.LowPart, end.HighPart));
+                    return out;
+                } else {
+                    return {};
+                }
+            }
+            return out;
+        }
+        return {};
     }
-    return {};
 }
 #endif
 
@@ -482,7 +549,7 @@ bool FileSystem::isFileLocked(const QString &fileName, LockMode mode)
     const auto handle = lockFile(fileName, mode);
     if (!handle) {
         const auto error = GetLastError();
-        if (error == ERROR_SHARING_VIOLATION) {
+        if (error == ERROR_SHARING_VIOLATION || error == ERROR_LOCK_VIOLATION) {
             return true;
         } else if (error != ERROR_FILE_NOT_FOUND && error != ERROR_PATH_NOT_FOUND) {
             qCWarning(lcFileSystem()) << Q_FUNC_INFO << Utility::formatWinError(error);
@@ -518,38 +585,88 @@ bool FileSystem::isJunction(const QString &filename)
 #endif
 }
 
-#ifdef Q_OS_WIN
-QString FileSystem::pathtoUNC(const QString &_str)
-{
-    Q_ASSERT(QFileInfo(_str).isAbsolute());
-    if (_str.isEmpty()) {
-        return _str;
-    }
-    const QString str = QDir::toNativeSeparators(_str);
-    const QLatin1Char sep('\\');
-
-    // we already have a unc path
-    if (str.startsWith(sep + sep)) {
-        return str;
-    }
-    // prepend \\?\ and to support long names
-
-    if (str.at(0) == sep) {
-        // should not happen as we require the path to be absolute
-        return QStringLiteral("\\\\?") + str;
-    }
-    return QStringLiteral("\\\\?\\") + str;
-}
-#endif
-
-bool FileSystem::isChildPathOf(const QString &child, const QString &parent)
+bool FileSystem::isChildPathOf(QStringView child, QStringView parent)
 {
     // if it is a relative path assume a local file, resolve it based on root
     const auto sensitivity = Utility::fsCaseSensitivity();
-    return (child.startsWith(parent.endsWith(QLatin1Char('/')) ? parent : parent + QLatin1Char('/'), sensitivity)
-        // clear trailing slashes etc
-        || QString::compare(QDir::cleanPath(parent), QDir::cleanPath(child), sensitivity) == 0);
+
+    // Fast-path the 3 common cases in this if-else statement:
+    if (parent.isEmpty()) {
+        // The empty parent is often used as the sync root, as (child) items do not start with a `/`
+        return true;
+    }
+
+    const auto isSeparator = [](QChar c) { return c == QLatin1Char('/') || (Utility::isWindows() && c == QLatin1Char('\\')); };
+
+    if (isSeparator(parent.back())) {
+        // Here we can do a normal prefix check, because the parent is "terminated" with a slash,
+        // and we can't walk into the case in the else below.
+        if (child.startsWith(parent, sensitivity)) {
+            return true;
+        }
+        // else: do the `cleanPath` version below
+    } else {
+        // Here we fast-path for the case the child is "Documents/a" and the parent is "Documents",
+        // and *not* the case where the child is "Documents/a" and the parent is "Document". In
+        // the latter case, the does start with the name of the parent, but there is an extra 's',
+        // and not a slash.
+
+        // Prevent a string concatenation like in `child.startsWith(parent + QLatin1Char('/'))`:
+        // first check if the child string starts with the parent string
+        if (child.startsWith(parent, sensitivity)) {
+            // ok, now check if the character after the parent is a '/'
+            if (child.length() >= parent.length() + 1 && isSeparator(child.at(parent.length()))) {
+                return true;
+            }
+            // else: do the `cleanPath` version below
+        }
+    }
+
+    // Slow path (`QDir::cleanPath` does lots of string operations):
+    const QString cleanParent = QDir::cleanPath(parent.toString());
+    const QString cleanChild = QDir::cleanPath(child.toString());
+    // cleanPath removes trailing slashes, add one to parent to be sure we handle a child path
+    // /root/foo/bar is not  a child of /root/fo, therefor the trailing slash is important
+    if (cleanChild.startsWith(cleanParent + QLatin1Char('/'), sensitivity)) {
+        return true;
+    }
+    // both paths are the same /root/foo == /root/foo
+    if (cleanChild.compare(cleanParent, sensitivity) == 0) {
+        return true;
+    }
+    return false;
 }
+
+QString FileSystem::createPortableFileName(const QString &path, const QString &fileName, qsizetype reservedSize)
+{
+    // remove leading and trailing whitespace
+    QString tmp = pathEscape(fileName);
+    // limit size to fileNameMaxC
+    tmp.resize(std::min<qsizetype>(tmp.size(), fileNameMaxC - reservedSize));
+    // remove eventual trailing whitespace after the resize
+    tmp = tmp.trimmed();
+
+    return QDir::cleanPath(path + QLatin1Char('/') + tmp);
+}
+
+QString FileSystem::pathEscape(const QString &s)
+{
+    QString tmp = s;
+    tmp.replace(QLatin1String("../"), QString(replacementCharC));
+#ifdef Q_OS_WIN
+    tmp.replace(QLatin1String("..\\"), QString(replacementCharC));
+#endif
+
+    // escape the folder separator, "\" is handled in IllegalFilenameCharsWindows
+    tmp.replace(QLatin1Char('/'), replacementCharC);
+
+    // replace non portable characters
+    for (auto c : IllegalFilenameCharsWindows) {
+        tmp.replace(c, replacementCharC);
+    }
+    return tmp.trimmed();
+}
+
 } // namespace OCC
 
 #include "moc_filesystembase.cpp"
