@@ -17,6 +17,7 @@
 #include "accountmanager.h"
 #include "application.h"
 #include "configfile.h"
+#include "fetchserversettings.h"
 
 #include "libsync/creds/abstractcredentials.h"
 #include "libsync/creds/httpcredentials.h"
@@ -35,6 +36,9 @@
 #include <QRandomGenerator>
 #include <QSettings>
 #include <QTimer>
+#if QT_VERSION >= QT_VERSION_CHECK(6, 3, 0)
+#include <QNetworkInformation>
+#endif
 
 using namespace std::chrono;
 using namespace std::chrono_literals;
@@ -110,6 +114,38 @@ AccountState::AccountState(AccountPtr account)
         },
         Qt::QueuedConnection);
 
+#if QT_VERSION >= QT_VERSION_CHECK(6, 3, 0)
+    if (QNetworkInformation::loadDefaultBackend()) {
+        connect(QNetworkInformation::instance(), &QNetworkInformation::reachabilityChanged, this, [this](QNetworkInformation::Reachability reachability) {
+            switch (reachability) {
+            case QNetworkInformation::Reachability::Online:
+                [[fallthrough]];
+            case QNetworkInformation::Reachability::Site:
+                [[fallthrough]];
+            case QNetworkInformation::Reachability::Unknown:
+                // the connection might not yet be established
+                QTimer::singleShot(0, this, [this] { checkConnectivity(false); });
+                break;
+            case QNetworkInformation::Reachability::Disconnected:
+                // explicitly set disconnected, this way a successful checkConnectivity call above will trigger a local discover
+                if (state() != State::SignedOut) {
+                    setState(State::Disconnected);
+                }
+                [[fallthrough]];
+            case QNetworkInformation::Reachability::Local:
+                break;
+            }
+        });
+    } else {
+        qCWarning(lcAccountState) << "Failed to load QNetworkInformation";
+    }
+#endif
+    // as a fallback and to recover after server issues we also poll
+    auto timer = new QTimer(this);
+    timer->setInterval(ConnectionValidator::DefaultCallingInterval);
+    connect(timer, &QTimer::timeout, this, [this] { checkConnectivity(false); });
+    timer->start();
+
     connect(account->credentials(), &AbstractCredentials::requestLogout, this, [this] {
         setState(State::SignedOut);
     });
@@ -117,18 +153,20 @@ AccountState::AccountState(AccountPtr account)
     if (FolderMan::instance()) {
         FolderMan::instance()->socketApi()->registerAccount(account);
     }
+
+    connect(account.data(), &Account::appProviderErrorOccured, this, [](const QString &error) {
+        QMessageBox *msgBox = new QMessageBox(QMessageBox::Information, Theme::instance()->appNameGUI(), error, {}, ocApp()->gui()->settingsDialog());
+        msgBox->setAttribute(Qt::WA_DeleteOnClose);
+        msgBox->open();
+        ocApp()->gui()->raiseDialog(msgBox);
+    });
 }
 
-AccountState::~AccountState()
-{
-    if (FolderMan::instance()) {
-        FolderMan::instance()->socketApi()->unregisterAccount(account());
-    }
-}
+AccountState::~AccountState() { }
 
-AccountStatePtr AccountState::loadFromSettings(AccountPtr account, const QSettings &settings)
+std::unique_ptr<AccountState> AccountState::loadFromSettings(AccountPtr account, const QSettings &settings)
 {
-    auto accountState = AccountStatePtr(new AccountState(account));
+    auto accountState = std::unique_ptr<AccountState>(new AccountState(account));
     const bool userExplicitlySignedOut = settings.value(userExplicitlySignedOutC(), false).toBool();
     if (userExplicitlySignedOut) {
         // see writeToSettings below
@@ -138,9 +176,9 @@ AccountStatePtr AccountState::loadFromSettings(AccountPtr account, const QSettin
     return accountState;
 }
 
-AccountStatePtr AccountState::fromNewAccount(AccountPtr account)
+std::unique_ptr<AccountState> AccountState::fromNewAccount(AccountPtr account)
 {
-    return AccountStatePtr(new AccountState(account));
+    return std::unique_ptr<AccountState>(new AccountState(account));
 }
 
 void AccountState::writeToSettings(QSettings &settings) const
@@ -196,16 +234,22 @@ void AccountState::setState(State state)
             _connectionValidator.clear();
             checkConnectivity();
         }
-        if (oldState == Connected || _state == Connected) {
-            emit isConnectedChanged();
-        }
     }
 
     // might not have changed but the underlying _connectionErrors might have
     if (_state == Connected) {
-        QTimer::singleShot(0, this, [this] {
+        QTimer::singleShot(0, this, [this, oldState] {
             // ensure the connection validator is done
             _queueGuard.unblock();
+            // update capabilites and fetch relevant settings
+            _fetchCapabilitiesJob = new FetchServerSettingsJob(account(), this);
+            connect(_fetchCapabilitiesJob.get(), &FetchServerSettingsJob::finishedSignal, this, [oldState, this] {
+                if (oldState == Connected || _state == Connected) {
+                    _fetchCapabilitiesJob.clear();
+                    emit isConnectedChanged();
+                }
+            });
+            _fetchCapabilitiesJob->start();
         });
     }
     // don't anounce a state change from connected to connected
@@ -258,25 +302,27 @@ void AccountState::tagLastSuccessfullETagRequest(const QDateTime &tp)
 
 void AccountState::checkConnectivity(bool blockJobs)
 {
-    if (_state != Connected) {
-        setState(Connecting);
-    }
-    qCWarning(lcAccountState) << "checkConnectivity blocking:" << blockJobs;
     if (isSignedOut() || _waitingForNewCredentials) {
         return;
     }
+    qCInfo(lcAccountState) << "checkConnectivity blocking:" << blockJobs << account()->displayName();
+    if (_state != Connected) {
+        setState(Connecting);
+    }
     if (_tlsDialog) {
         qCDebug(lcAccountState) << "Skip checkConnectivity, waiting for tls dialog";
         return;
     }
 
+
     if (_connectionValidator && blockJobs && !_queueGuard.queue()->isBlocked()) {
         // abort already running non blocking validator
         _connectionValidator->deleteLater();
         _connectionValidator.clear();
     }
     if (_connectionValidator) {
-        qCWarning(lcAccountState) << "ConnectionValidator already running, ignoring" << account()->displayName();
+        qCWarning(lcAccountState) << "ConnectionValidator already running, ignoring" << account()->displayName()
+                                  << "Queue is blocked:" << _queueGuard.queue()->isBlocked();
         return;
     }
 
@@ -289,6 +335,7 @@ void AccountState::checkConnectivity(bool blockJobs)
     if (account()->hasCapabilities()) {
         // IF the account is connected the connection check can be skipped
         // if the last successful etag check job is not so long ago.
+        // TODO: https://github.com/owncloud/client/issues/10935
         const auto pta = account()->capabilities().remotePollInterval();
         const auto polltime = duration_cast<seconds>(ConfigFile().remotePollInterval(pta));
         const auto elapsed = _timeOfLastETagCheck.secsTo(QDateTime::currentDateTimeUtc());
@@ -321,7 +368,11 @@ void AccountState::checkConnectivity(bool blockJobs)
                 connect(_tlsDialog, &TlsErrorDialog::accepted, _tlsDialog, [certs, blockJobs, this]() {
                     _account->addApprovedCerts(certs);
                     _tlsDialog.clear();
-                    _waitingForNewCredentials = false;
+                    // force a new _connectionValidator
+                    if (_connectionValidator) {
+                        _connectionValidator->deleteLater();
+                        _connectionValidator.clear();
+                    }
                     checkConnectivity(blockJobs);
                 });
                 connect(_tlsDialog, &TlsErrorDialog::rejected, this, [certs, this]() {
@@ -475,7 +526,7 @@ void AccountState::slotInvalidCredentials()
     }
 }
 
-void AccountState::slotCredentialsFetched(AbstractCredentials *)
+void AccountState::slotCredentialsFetched()
 {
     // Make a connection attempt, no matter whether the credentials are
     // ready or not - we want to check whether we can get an SSL connection
@@ -486,14 +537,13 @@ void AccountState::slotCredentialsFetched(AbstractCredentials *)
     checkConnectivity();
 }
 
-void AccountState::slotCredentialsAsked(AbstractCredentials *credentials)
+void AccountState::slotCredentialsAsked()
 {
-    qCInfo(lcAccountState) << "Credentials asked for" << _account->url().toString()
-                           << "are they ready?" << credentials->ready();
+    qCInfo(lcAccountState) << "Credentials asked for" << _account->url().toString() << "are they ready?" << _account->credentials()->ready();
 
     _waitingForNewCredentials = false;
 
-    if (!credentials->ready()) {
+    if (!_account->credentials()->ready()) {
         // User canceled the connection or did not give a password
         setState(SignedOut);
         return;
@@ -531,4 +581,21 @@ QuotaInfo *AccountState::quotaInfo()
     return _quotaInfo;
 }
 
+bool AccountState::isSettingUp() const
+{
+    return _settingUp;
+}
+
+void AccountState::setSettingUp(bool settingUp)
+{
+    if (_settingUp != settingUp) {
+        _settingUp = settingUp;
+        Q_EMIT isSettingUpChanged();
+    }
+}
+bool AccountState::readyForSync() const
+{
+    return !_fetchCapabilitiesJob && isConnected();
+}
+
 } // namespace OCC
