@@ -18,8 +18,15 @@
 #include "common/asserts.h"
 #include "folderman.h"
 #include "folderstatusdelegate.h"
+#include "gui/quotainfo.h"
 #include "theme.h"
 
+#include "resources/resources.h"
+
+#include "libsync/graphapi/space.h"
+#include "libsync/graphapi/spacesmanager.h"
+
+#include <QDir>
 #include <QFileIconProvider>
 #include <QVarLengthArray>
 
@@ -39,14 +46,39 @@ namespace {
 
     const char propertyParentIndexC[] = "oc_parentIndex";
     const char propertyPermissionMap[] = "oc_permissionMap";
-}
 
-static QString removeTrailingSlash(const QString &s)
-{
-    if (s.endsWith('/')) {
-        return s.left(s.size() - 1);
+    int64_t getQuota(const AccountStatePtr &accountState, const QString &spaceId, FolderStatusModel::Columns type)
+    {
+        if (auto spacesManager = accountState->account()->spacesManager()) {
+            const auto *space = spacesManager->space(spaceId);
+            if (space) {
+                const auto quota = space->drive().getQuota();
+                if (quota.isValid()) {
+                    switch (type) {
+                    case FolderStatusModel::Columns::QuotaTotal:
+                        return quota.getTotal();
+                    case FolderStatusModel::Columns::QuotaUsed:
+                        return quota.getUsed();
+                    default:
+                        Q_UNREACHABLE();
+                    }
+                }
+            }
+        }
+        return {};
+    }
+
+    int64_t getQuotaOc10(const AccountStatePtr &accountState, const QUrl &davUrl, FolderStatusModel::Columns type)
+    {
+        switch (type) {
+        case FolderStatusModel::Columns::QuotaTotal:
+            return accountState->quotaInfo()->lastQuotaTotalBytes();
+        case FolderStatusModel::Columns::QuotaUsed:
+            return accountState->quotaInfo()->lastQuotaUsedBytes();
+        default:
+            Q_UNREACHABLE();
+        }
     }
-    return s;
 }
 
 FolderStatusModel::FolderStatusModel(QObject *parent)
@@ -54,60 +86,59 @@ FolderStatusModel::FolderStatusModel(QObject *parent)
     , _accountState(nullptr)
     , _dirty(false)
 {
+    connect(this, &FolderStatusModel::rowsInserted, this, &FolderStatusModel::dirtyChanged);
 }
 
 FolderStatusModel::~FolderStatusModel()
 {
 }
 
-static bool sortByFolderHeader(const FolderStatusModel::SubFolderInfo &lhs, const FolderStatusModel::SubFolderInfo &rhs)
-{
-    return QString::compare(lhs._folder->shortGuiRemotePathOrAppName(),
-               rhs._folder->shortGuiRemotePathOrAppName(),
-               Qt::CaseInsensitive)
-        < 0;
-}
-
-void FolderStatusModel::setAccountState(const AccountState *accountState)
+void FolderStatusModel::setAccountState(const AccountStatePtr &accountState)
 {
     beginResetModel();
     _dirty = false;
     _folders.clear();
-    _accountState = accountState;
+    if (_accountState != accountState) {
+        Q_ASSERT(!_accountState);
+        _accountState = accountState;
 
-    connect(FolderMan::instance(), &FolderMan::folderSyncStateChange,
-        this, &FolderStatusModel::slotFolderSyncStateChange, Qt::UniqueConnection);
-    connect(FolderMan::instance(), &FolderMan::scheduleQueueChanged,
-        this, &FolderStatusModel::slotFolderScheduleQueueChanged, Qt::UniqueConnection);
+        connect(FolderMan::instance(), &FolderMan::folderSyncStateChange, this, &FolderStatusModel::slotFolderSyncStateChange);
 
-    for (const auto &f : FolderMan::instance()->map()) {
+        if (accountState->supportsSpaces()) {
+            connect(accountState->account()->spacesManager(), &GraphApi::SpacesManager::updated, this, [this] {
+                beginResetModel();
+                endResetModel();
+            });
+        }
+    }
+    for (const auto &f : FolderMan::instance()->folders()) {
         if (!accountState)
             break;
         if (f->accountState() != accountState)
             continue;
         SubFolderInfo info;
-        info._name = f->alias();
-        info._path = "/";
+        // the name is not actually used with the root element
+        info._name = QLatin1Char('/');
+        info._path = QLatin1Char('/');
         info._folder = f;
         info._checked = Qt::PartiallyChecked;
         _folders << info;
 
-        connect(f, &Folder::progressInfo, this, [f, this](const ProgressInfo &info) {
-            slotSetProgress(info, f);
+        connect(ProgressDispatcher::instance(), &ProgressDispatcher::progressInfo, this, [f, this](Folder *folder, const ProgressInfo &progress) {
+            if (folder == f) {
+                slotSetProgress(progress, f);
+            }
         });
 
         connect(f, &Folder::newBigFolderDiscovered, this, &FolderStatusModel::slotNewBigFolder, Qt::UniqueConnection);
     }
 
-    // Sort by header text
-    std::sort(_folders.begin(), _folders.end(), sortByFolderHeader);
-
     // Set the root _pathIdx after the sorting
     for (int i = 0; i < _folders.size(); ++i) {
         _folders[i]._pathIdx << i;
     }
 
-    endResetModel();
+    Q_EMIT endResetModel();
     emit dirtyChanged();
 }
 
@@ -115,7 +146,7 @@ void FolderStatusModel::setAccountState(const AccountState *accountState)
 Qt::ItemFlags FolderStatusModel::flags(const QModelIndex &index) const
 {
     if (!_accountState) {
-        return nullptr;
+        return Qt::NoItemFlags;
     }
 
     // Always enable the item. If it isn't enabled, it cannot be in the selection model, so all
@@ -124,14 +155,6 @@ Qt::ItemFlags FolderStatusModel::flags(const QModelIndex &index) const
     const auto flags = Qt::ItemIsEnabled;
 
     switch (classify(index)) {
-    case AddButton: {
-        Qt::ItemFlags ret;
-        ret = Qt::ItemNeverHasChildren;
-        if (!_accountState->isConnected()) {
-            return ret;
-        }
-        return flags | ret;
-    }
     case FetchLabel:
         return flags | Qt::ItemNeverHasChildren;
     case RootFolder:
@@ -150,41 +173,51 @@ QVariant FolderStatusModel::data(const QModelIndex &index, int role) const
     if (role == Qt::EditRole)
         return QVariant();
 
-    switch (classify(index)) {
-    case AddButton: {
-        if (role == FolderStatusDelegate::AddButton) {
-            return QVariant(true);
-        } else if (role == Qt::ToolTipRole) {
-            if (!_accountState->isConnected()) {
-                return tr("You need to be connected to add a folder");
-            }
-            return tr("Click this button to add a folder to synchronize.");
-        }
-        return QVariant();
+    const Columns column = static_cast<Columns>(index.column());
+    const auto itemType = classify(index);
+    switch (column) {
+    case Columns::ItemType:
+        return itemType;
+    case Columns::IsUsingSpaces:
+        return _accountState->supportsSpaces();
+    default:
+        break;
     }
+
+    switch (itemType) {
     case SubFolder: {
         const auto &x = static_cast<SubFolderInfo *>(index.internalPointer())->_subs.at(index.row());
+
         switch (role) {
         case Qt::DisplayRole:
-            //: Example text: "File.txt (23KB)"
-            return x._size < 0 ? x._name : tr("%1 (%2)").arg(x._name, Utility::octetsToString(x._size));
+            switch (column) {
+            case Columns::FolderPathRole: {
+                auto f = x._folder;
+                if (!f)
+                    return QVariant();
+                return QVariant(f->path() + x._path);
+            }
+            case Columns::IsReady:
+                return x._folder->isReady();
+            case Columns::HeaderRole:
+                return x._size < 0 ? x._name : tr("%1 (%2)", "filename (size)").arg(x._name, Utility::octetsToString(x._size));
+            default:
+                return {};
+            }
         case Qt::ToolTipRole:
-            return QString(QLatin1String("<qt>") + Utility::escape(x._size < 0 ? x._name : tr("%1 (%2)").arg(x._name, Utility::octetsToString(x._size))) + QLatin1String("</qt>"));
+            return QString(QLatin1String("<qt>") + Utility::escape(x._size < 0 ? x._name : tr("%1 (%2)", "filename (size)").arg(x._name, Utility::octetsToString(x._size))) + QLatin1String("</qt>"));
         case Qt::CheckStateRole:
             return x._checked;
         case Qt::DecorationRole:
-            return QFileIconProvider().icon(x._isExternal ? QFileIconProvider::Network : QFileIconProvider::Folder);
+            if (x._isExternal) {
+                return QFileIconProvider().icon(QFileIconProvider::Network);
+            }
+            return Resources::getCoreIcon(QStringLiteral("folder-sync"));
         case Qt::ForegroundRole:
             if (x._isUndecided) {
                 return QColor(Qt::red);
             }
             break;
-        case FolderStatusDelegate::FolderPathRole: {
-            auto f = x._folder;
-            if (!f)
-                return QVariant();
-            return QVariant(f->path() + x._path);
-        }
         }
     }
         return QVariant();
@@ -194,7 +227,7 @@ QVariant FolderStatusModel::data(const QModelIndex &index, int role) const
         case Qt::DisplayRole:
             if (x->_hasError) {
                 return QVariant(tr("Error while loading the list of folders from the server.")
-                    + QString("\n") + x->_lastErrorString);
+                    + QStringLiteral("\n") + x->_lastErrorString);
             } else {
                 return tr("Fetching folder list from server...");
             }
@@ -215,74 +248,111 @@ QVariant FolderStatusModel::data(const QModelIndex &index, int role) const
     const SubFolderInfo::Progress &progress = folderInfo._progress;
     const bool accountConnected = _accountState->isConnected();
 
-    switch (role) {
-    case FolderStatusDelegate::FolderPathRole:
-        return f->shortGuiLocalPath();
-    case FolderStatusDelegate::FolderSecondPathRole:
-        return f->remotePath();
-    case FolderStatusDelegate::FolderConflictMsg:
-        return (f->syncResult().hasUnresolvedConflicts())
-            ? QStringList(tr("There are unresolved conflicts. Click for details."))
-            : QStringList();
-    case FolderStatusDelegate::FolderErrorMsg: {
-        auto errors = f->syncResult().errorStrings();
-        const auto legacyError = FolderMan::instance()->unsupportedConfiguration(f->path());
-        if (!legacyError) {
-            // the error message might contain new lines, the delegate only expect multiple single line values
-            errors.append(legacyError.error().split(QLatin1Char('\n')));
+    const auto getSpace = [&]() -> GraphApi::Space * {
+        if (_accountState->supportsSpaces()) {
+            return _accountState->account()->spacesManager()->space(f->spaceId());
         }
-        return errors;
-    }
-    case FolderStatusDelegate::FolderInfoMsg:
-        return f->isReady() && f->virtualFilesEnabled() && f->vfs().mode() != Vfs::Mode::WindowsCfApi
-            ? QStringList(tr("Virtual file support is enabled."))
-            : QStringList();
-    case FolderStatusDelegate::SyncRunning:
-        return f->syncResult().status() == SyncResult::SyncRunning;
-    case FolderStatusDelegate::HeaderRole:
-        return f->shortGuiRemotePathOrAppName();
-    case FolderStatusDelegate::FolderAliasRole:
-        return f->alias();
-    case FolderStatusDelegate::FolderSyncPaused:
-        return f->syncPaused();
-    case FolderStatusDelegate::FolderAccountConnected:
-        return accountConnected;
+        return nullptr;
+    };
+
+    switch (role) {
+    case Qt::DisplayRole:
+        switch (column) {
+        case Columns::FolderPathRole:
+            return f->shortGuiLocalPath();
+        case Columns::FolderSecondPathRole:
+            return f->remotePath();
+        case Columns::FolderConflictMsg:
+            return (f->syncResult().hasUnresolvedConflicts())
+                ? QStringList(tr("There are unresolved conflicts. Click for details."))
+                : QStringList();
+        case Columns::FolderErrorMsg: {
+            auto errors = f->syncResult().errorStrings();
+            const auto legacyError = FolderMan::instance()->unsupportedConfiguration(f->path());
+            if (!legacyError) {
+                // the error message might contain new lines, the delegate only expect multiple single line values
+                errors.append(legacyError.error().split(QLatin1Char('\n')));
+            }
+            return errors;
+        }
+        case Columns::FolderInfoMsg:
+            return f->isReady() && f->virtualFilesEnabled() && f->vfs().mode() != Vfs::Mode::WindowsCfApi ? QStringList(tr("Virtual file support is enabled."))
+                                                                                                          : QStringList();
+        case Columns::SyncRunning:
+            return f->syncResult().status() == SyncResult::SyncRunning;
+        case Columns::HeaderRole: {
+            if (auto *space = getSpace()) {
+                return space->displayName();
+            }
+            return f->displayName();
+        }
+        case Columns::FolderImage:
+            if (auto *space = getSpace()) {
+                return space->image();
+            }
+            return Resources::getCoreIcon(QStringLiteral("folder-sync"));
+        case Columns::FolderSyncPaused:
+            return f->syncPaused();
+        case Columns::FolderAccountConnected:
+            return accountConnected;
+        case Columns::FolderStatusIconRole: {
+            auto status = f->syncResult();
+            if (!accountConnected) {
+                status.setStatus(SyncResult::Status::Offline);
+            } else if (f->syncPaused()) {
+                status.setStatus(SyncResult::Status::Paused);
+            }
+            return Theme::instance()->syncStateIconName(status);
+        }
+        case Columns::SyncProgressItemString:
+            return progress._progressString;
+        case Columns::WarningCount:
+            return progress._warningCount;
+        case Columns::SyncProgressOverallPercent:
+            return progress._overallPercent;
+        case Columns::SyncProgressOverallString:
+            return progress._overallSyncString;
+        case Columns::FolderSyncText: {
+            if (auto *space = getSpace()) {
+                if (!space->drive().getDescription().isEmpty()) {
+                    return space->drive().getDescription();
+                }
+            }
+            return tr("Local folder: %1").arg(f->shortGuiLocalPath());
+        }
+        case Columns::IsReady:
+            return f->isReady();
+        case Columns::IsDeployed:
+            return f->isDeployed();
+        case Columns::Priority:
+            return f->priority();
+        case Columns::QuotaTotal:
+            [[fallthrough]];
+        case Columns::QuotaUsed:
+            if (_accountState->supportsSpaces()) {
+                return QVariant::fromValue(getQuota(_accountState, f->spaceId(), column));
+            } else {
+                return QVariant::fromValue(getQuotaOc10(_accountState, f->webDavUrl(), column));
+            }
+        case Columns::IsUsingSpaces: // handled before
+            [[fallthrough]];
+        case Columns::ItemType: // handled before
+            [[fallthrough]];
+        case Columns::ColumnCount:
+            Q_UNREACHABLE();
+            break;
+        }
+        break;
     case Qt::ToolTipRole: {
-        QString toolTip;
         if (!progress.isNull()) {
             return progress._progressString;
         }
-        if (accountConnected)
-            toolTip = Theme::instance()->statusHeaderText(f->syncResult().status());
-        else
-            toolTip = tr("Signed out");
-        toolTip += "\n";
-        toolTip += folderInfo._folder->path();
-        return toolTip;
-    }
-    case FolderStatusDelegate::FolderStatusIconRole:
         if (accountConnected) {
-            auto theme = Theme::instance();
-            if (f->syncPaused()) {
-                return theme->folderDisabledIcon();
-            } else {
-                return theme->syncStateIcon(f->syncResult().status());
-            }
+            return tr("%1\n%2").arg(Utility::enumToDisplayName(f->syncResult().status()), QDir::toNativeSeparators(folderInfo._folder->path()));
         } else {
-            return Theme::instance()->folderOfflineIcon();
+            return tr("Signed out\n%1").arg(QDir::toNativeSeparators(folderInfo._folder->path()));
         }
-    case FolderStatusDelegate::SyncProgressItemString:
-        return progress._progressString;
-    case FolderStatusDelegate::WarningCount:
-        return progress._warningCount;
-    case FolderStatusDelegate::SyncProgressOverallPercent:
-        return progress._overallPercent;
-    case FolderStatusDelegate::SyncProgressOverallString:
-        return progress._overallSyncString;
-    case FolderStatusDelegate::FolderSyncText:
-        return tr("Local folder: %1").arg(f->shortGuiLocalPath());
-    case FolderStatusDelegate::IsReady:
-        return f->isReady();
+    }
     }
     return QVariant();
 }
@@ -353,7 +423,7 @@ bool FolderStatusModel::setData(const QModelIndex &index, const QVariant &value,
         }
         _dirty = true;
         emit dirtyChanged();
-        emit dataChanged(index, index, QVector<int>() << role);
+        emit dataChanged(index, index, { role });
         return true;
     }
     return QAbstractItemModel::setData(index, value, role);
@@ -362,17 +432,13 @@ bool FolderStatusModel::setData(const QModelIndex &index, const QVariant &value,
 
 int FolderStatusModel::columnCount(const QModelIndex &) const
 {
-    return 1;
+    return static_cast<int>(Columns::ColumnCount);
 }
 
 int FolderStatusModel::rowCount(const QModelIndex &parent) const
 {
     if (!parent.isValid()) {
-        if (Theme::instance()->singleSyncFolder() && _folders.count() != 0) {
-            // "Add folder" button not visible in the singleSyncFolder configuration.
-            return _folders.count();
-        }
-        return _folders.count() + 1; // +1 for the "add folder" button
+        return _folders.count();
     }
     auto info = infoForIndex(parent);
     if (!info)
@@ -391,10 +457,8 @@ FolderStatusModel::ItemType FolderStatusModel::classify(const QModelIndex &index
             return SubFolder;
         }
     }
-    if (index.row() < _folders.count()) {
-        return RootFolder;
-    }
-    return AddButton;
+
+    return RootFolder;
 }
 
 FolderStatusModel::SubFolderInfo *FolderStatusModel::infoForIndex(const QModelIndex &index) const
@@ -424,7 +488,7 @@ QModelIndex FolderStatusModel::indexForPath(Folder *f, const QString &path) cons
         return QModelIndex();
     }
 
-    int slashPos = path.lastIndexOf('/');
+    int slashPos = path.lastIndexOf(QLatin1Char('/'));
     if (slashPos == -1) {
         // first level folder
         for (int i = 0; i < _folders.size(); ++i) {
@@ -473,7 +537,6 @@ QModelIndex FolderStatusModel::index(int row, int column, const QModelIndex &par
         return createIndex(row, column /*, nullptr*/);
     }
     switch (classify(parent)) {
-    case AddButton:
     case FetchLabel:
         return QModelIndex();
     case RootFolder:
@@ -501,9 +564,9 @@ QModelIndex FolderStatusModel::parent(const QModelIndex &child) const
     }
     switch (classify(child)) {
     case RootFolder:
-    case AddButton:
         return QModelIndex();
     case SubFolder:
+        [[fallthrough]];
     case FetchLabel:
         break;
     }
@@ -570,9 +633,16 @@ bool FolderStatusModel::canFetchMore(const QModelIndex &parent) const
 
 void FolderStatusModel::fetchMore(const QModelIndex &parent)
 {
-    if (!data(parent, FolderStatusDelegate::IsReady).toBool()) {
-        return;
+    {
+        const auto isReady = parent.siblingAtColumn(static_cast<int>(Columns::IsReady)).data();
+
+        Q_ASSERT(isReady.isValid());
+
+        if (!isReady.toBool()) {
+            return;
+        }
     }
+
     auto info = infoForIndex(parent);
 
     if (!info || info->_fetched || info->_fetchingJob)
@@ -582,17 +652,15 @@ void FolderStatusModel::fetchMore(const QModelIndex &parent)
     if (info->_path != QLatin1String("/")) {
         path += info->_path;
     }
-    LsColJob *job = new LsColJob(_accountState->account(), path, this);
+    PropfindJob *job = new PropfindJob(_accountState->account(), info->_folder->webDavUrl(), path, PropfindJob::Depth::One, this);
     info->_fetchingJob = job;
-    job->setProperties(QList<QByteArray>() << "resourcetype"
-                                           << "http://owncloud.org/ns:size"
-                                           << "http://owncloud.org/ns:permissions");
-    job->setTimeout(60 * 1000);
-    connect(job, &LsColJob::directoryListingSubfolders,
+    job->setProperties({ QByteArrayLiteral("resourcetype"), QByteArrayLiteral("http://owncloud.org/ns:size"), QByteArrayLiteral("http://owncloud.org/ns:permissions") });
+    job->setTimeout(60s);
+    connect(job, &PropfindJob::directoryListingSubfolders,
         this, &FolderStatusModel::slotUpdateDirectories);
-    connect(job, &LsColJob::finishedWithError,
+    connect(job, &PropfindJob::finishedWithError,
         this, &FolderStatusModel::slotLscolFinishedWithError);
-    connect(job, &LsColJob::directoryListingIterated,
+    connect(job, &PropfindJob::directoryListingIterated,
         this, &FolderStatusModel::slotGatherPermissions);
 
     job->start();
@@ -602,7 +670,7 @@ void FolderStatusModel::fetchMore(const QModelIndex &parent)
 
     // Show 'fetching data...' hint after a while.
     _fetchingItems[persistentIndex].start();
-    QTimer::singleShot(1000, this, &FolderStatusModel::slotShowFetchProgress);
+    QTimer::singleShot(1s, this, &FolderStatusModel::slotShowFetchProgress);
 }
 
 void FolderStatusModel::resetAndFetch(const QModelIndex &parent)
@@ -614,7 +682,7 @@ void FolderStatusModel::resetAndFetch(const QModelIndex &parent)
 
 void FolderStatusModel::slotGatherPermissions(const QString &href, const QMap<QString, QString> &map)
 {
-    auto it = map.find("permissions");
+    auto it = map.find(QStringLiteral("permissions"));
     if (it == map.end())
         return;
 
@@ -628,7 +696,7 @@ void FolderStatusModel::slotGatherPermissions(const QString &href, const QMap<QS
 
 void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
 {
-    auto job = qobject_cast<LsColJob *>(sender());
+    auto job = qobject_cast<PropfindJob *>(sender());
     OC_ASSERT(job);
     QModelIndex idx = qvariant_cast<QPersistentModelIndex>(job->property(propertyParentIndexC));
     auto parentInfo = infoForIndex(idx);
@@ -654,16 +722,20 @@ void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
 
     QUrl url = parentInfo->_folder->remoteUrl();
     QString pathToRemove = url.path();
-    if (!pathToRemove.endsWith('/'))
-        pathToRemove += '/';
+    if (!pathToRemove.endsWith(QLatin1Char('/')))
+        pathToRemove += QLatin1Char('/');
 
-    QStringList selectiveSyncBlackList;
+    QSet<QString> selectiveSyncBlackList;
     bool ok1 = true;
     bool ok2 = true;
     if (parentInfo->_checked == Qt::PartiallyChecked) {
         selectiveSyncBlackList = parentInfo->_folder->journalDb()->getSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, &ok1);
     }
-    auto selectiveSyncUndecidedList = parentInfo->_folder->journalDb()->getSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, &ok2);
+    // qlist for historic reasons
+    QStringList selectiveSyncUndecidedList = [&] {
+        const auto list = parentInfo->_folder->journalDb()->getSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, &ok2);
+        return QStringList{list.cbegin(), list.cend()};
+    }();
 
     if (!(ok1 && ok2)) {
         qCWarning(lcFolderStatus) << "Could not retrieve selective sync info from journal";
@@ -698,9 +770,9 @@ void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
         newInfo._pathIdx = parentInfo->_pathIdx;
         newInfo._pathIdx << newSubs.size();
         newInfo._size = job->sizes().value(path);
-        newInfo._isExternal = permissionMap.value(removeTrailingSlash(path)).toString().contains("M");
+        newInfo._isExternal = permissionMap.value(Utility::stripTrailingSlash(path)).toString().contains(QLatin1String("M"));
         newInfo._path = relativePath;
-        newInfo._name = removeTrailingSlash(relativePath).split('/').last();
+        newInfo._name = Utility::stripTrailingSlash(relativePath).split(QLatin1Char('/')).last();
 
         if (relativePath.isEmpty())
             continue;
@@ -739,28 +811,29 @@ void FolderStatusModel::slotUpdateDirectories(const QStringList &list)
     }
 
     if (!newSubs.isEmpty()) {
-        beginInsertRows(idx, 0, newSubs.size() - 1);
+        Q_EMIT beginInsertRows(idx, 0, newSubs.size() - 1);
         parentInfo->_subs = std::move(newSubs);
-        endInsertRows();
+        Q_EMIT endInsertRows();
     }
 
     for (auto it = undecidedIndexes.begin(); it != undecidedIndexes.end(); ++it) {
-        suggestExpand(index(*it, 0, idx));
+        Q_EMIT suggestExpand(index(*it, 0, idx));
     }
     /* Try to remove the the undecided lists the items that are not on the server. */
     auto it = std::remove_if(selectiveSyncUndecidedList.begin(), selectiveSyncUndecidedList.end(),
         [&](const QString &s) { return selectiveSyncUndecidedSet.count(s); });
     if (it != selectiveSyncUndecidedList.end()) {
         selectiveSyncUndecidedList.erase(it, selectiveSyncUndecidedList.end());
+
         parentInfo->_folder->journalDb()->setSelectiveSyncList(
-            SyncJournalDb::SelectiveSyncUndecidedList, selectiveSyncUndecidedList);
+            SyncJournalDb::SelectiveSyncUndecidedList, {selectiveSyncUndecidedList.cbegin(), selectiveSyncUndecidedList.cend()});
         emit dirtyChanged();
     }
 }
 
 void FolderStatusModel::slotLscolFinishedWithError(QNetworkReply *r)
 {
-    auto job = qobject_cast<LsColJob *>(sender());
+    auto job = qobject_cast<PropfindJob *>(sender());
     OC_ASSERT(job);
     QModelIndex idx = qvariant_cast<QPersistentModelIndex>(job->property(propertyParentIndexC));
     if (!idx.isValid()) {
@@ -785,19 +858,18 @@ void FolderStatusModel::slotLscolFinishedWithError(QNetworkReply *r)
     }
 }
 
-QStringList FolderStatusModel::createBlackList(const FolderStatusModel::SubFolderInfo &root,
-    const QStringList &oldBlackList) const
+QSet<QString> FolderStatusModel::createBlackList(const FolderStatusModel::SubFolderInfo &root, const QSet<QString> &oldBlackList) const
 {
     switch (root._checked) {
     case Qt::Unchecked:
-        return QStringList(root._path);
+        return {root._path};
     case Qt::Checked:
-        return QStringList();
+        return {};
     case Qt::PartiallyChecked:
         break;
     }
 
-    QStringList result;
+    QSet<QString> result;
     if (root._fetched) {
         for (int i = 0; i < root._subs.count(); ++i) {
             result += createBlackList(root._subs.at(i), oldBlackList);
@@ -828,26 +900,25 @@ void FolderStatusModel::slotApplySelectiveSync()
 {
     for (const auto &folderInfo : qAsConst(_folders)) {
         if (!folderInfo._fetched) {
-            folderInfo._folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, QStringList());
+            folderInfo._folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, {});
             continue;
         }
         const auto folder = folderInfo._folder;
 
         bool ok;
-        auto oldBlackList = folder->journalDb()->getSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, &ok);
+        auto oldBlackListSet = folder->journalDb()->getSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, &ok);
         if (!ok) {
             qCWarning(lcFolderStatus) << "Could not read selective sync list from db.";
             continue;
         }
-        QStringList blackList = createBlackList(folderInfo, oldBlackList);
-        folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, blackList);
+        auto blackListSet = createBlackList(folderInfo, oldBlackListSet);
+        folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, blackListSet);
 
-        auto blackListSet = blackList.toSet();
-        auto oldBlackListSet = oldBlackList.toSet();
 
         // The folders that were undecided or blacklisted and that are now checked should go on the white list.
         // The user confirmed them already just now.
-        QStringList toAddToWhiteList = ((oldBlackListSet + folder->journalDb()->getSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, &ok).toSet()) - blackListSet).toList();
+        QSet<QString> toAddToWhiteList =
+            ((oldBlackListSet + folder->journalDb()->getSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, &ok)) - blackListSet);
 
         if (!toAddToWhiteList.isEmpty()) {
             auto whiteList = folder->journalDb()->getSelectiveSyncList(SyncJournalDb::SelectiveSyncWhiteList, &ok);
@@ -857,7 +928,7 @@ void FolderStatusModel::slotApplySelectiveSync()
             }
         }
         // clear the undecided list
-        folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, QStringList());
+        folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, {});
 
         // do the sync if there were changes
         const auto changes = (oldBlackListSet - blackListSet) + (blackListSet - oldBlackListSet);
@@ -873,7 +944,7 @@ void FolderStatusModel::slotApplySelectiveSync()
             }
             // Also make sure we see the local file that had been ignored before
             folder->slotNextSyncFullLocalDiscovery();
-            FolderMan::instance()->scheduleFolder(folder);
+            FolderMan::instance()->scheduler()->enqueueFolder(folder);
         }
     }
 
@@ -894,38 +965,36 @@ void FolderStatusModel::slotSetProgress(const ProgressInfo &progress, Folder *f)
 
     auto *pi = &folder._progress;
 
-    const QVector<int> roles = { FolderStatusDelegate::SyncProgressItemString, FolderStatusDelegate::WarningCount, Qt::ToolTipRole };
+    if (progress.status() == ProgressInfo::Done && !progress._lastCompletedItem.isEmpty()
+        && Progress::isWarningKind(progress._lastCompletedItem._status)) {
+        pi->_warningCount++;
+    }
+    // depending on the use of virtual files or small files this slot might be called very often.
+    // throttle the model updates to prevent an needlessly high cpu usage used on ui updates.
+    if (folder._lastProgressUpdateStatus != progress.status() || (std::chrono::steady_clock::now() - folder._lastProgressUpdated > progressUpdateTimeOutC)) {
+        folder._lastProgressUpdateStatus = progress.status();
 
-    switch (progress.status()) {
-    case ProgressInfo::None:
-        Q_UNREACHABLE();
-    case ProgressInfo::Discovery:
-        if (!progress._currentDiscoveredRemoteFolder.isEmpty()) {
-            pi->_overallSyncString = tr("Checking for changes in remote '%1'").arg(progress._currentDiscoveredRemoteFolder);
-            emit dataChanged(index(folderIndex), index(folderIndex), roles);
-        } else if (!progress._currentDiscoveredLocalFolder.isEmpty()) {
-            pi->_overallSyncString = tr("Checking for changes in local '%1'").arg(progress._currentDiscoveredLocalFolder);
-            emit dataChanged(index(folderIndex), index(folderIndex), roles);
-        }
-        break;
-    case ProgressInfo::Reconcile:
-        pi->_overallSyncString = tr("Reconciling changes");
-        emit dataChanged(index(folderIndex), index(folderIndex), roles);
-        break;
-    case ProgressInfo::Propagation:
-        Q_FALLTHROUGH();
-    case ProgressInfo::Done:
-        if (!progress._lastCompletedItem.isEmpty()
-            && Progress::isWarningKind(progress._lastCompletedItem._status)) {
-            pi->_warningCount++;
-        }
-
-        // progress updates are expensive, throtle them
-        if (std::chrono::steady_clock::now() - folder._lastProgressUpdated > progressUpdateTimeOutC) {
+        switch (progress.status()) {
+        case ProgressInfo::None:
+            Q_UNREACHABLE();
+            break;
+        case ProgressInfo::Discovery:
+            if (!progress._currentDiscoveredRemoteFolder.isEmpty()) {
+                pi->_overallSyncString = tr("Checking for changes in remote '%1'").arg(progress._currentDiscoveredRemoteFolder);
+            } else if (!progress._currentDiscoveredLocalFolder.isEmpty()) {
+                pi->_overallSyncString = tr("Checking for changes in local '%1'").arg(progress._currentDiscoveredLocalFolder);
+            }
+            break;
+        case ProgressInfo::Reconcile:
+            pi->_overallSyncString = tr("Reconciling changes");
+            break;
+        case ProgressInfo::Propagation:
+            Q_FALLTHROUGH();
+        case ProgressInfo::Done:
             computeProgress(progress, pi);
-            folder._lastProgressUpdated = std::chrono::steady_clock::now();
-            emit dataChanged(index(folderIndex), index(folderIndex), roles);
         }
+        emit dataChanged(index(folderIndex), index(folderIndex));
+        folder._lastProgressUpdated = std::chrono::steady_clock::now();
     }
 }
 
@@ -952,13 +1021,7 @@ void FolderStatusModel::computeProgress(const ProgressInfo &progress, SubFolderI
             estimatedUpBw += progress.fileProgress(citm._item).estimatedBandwidth;
         }
         auto fileName = QFileInfo(citm._item._file).fileName();
-        if (allFilenames.length() > 0) {
-            //: Build a list of file names
-            allFilenames.append(tr(", '%1'").arg(fileName));
-        } else {
-            //: Argument is a file name
-            allFilenames.append(tr("'%1'").arg(fileName));
-        }
+        allFilenames.append(tr("'%1'").arg(fileName));
     }
     if (curItemProgress == -1) {
         curItemProgress = curItem._size;
@@ -979,7 +1042,7 @@ void FolderStatusModel::computeProgress(const ProgressInfo &progress, SubFolderI
                     Utility::octetsToString(estimatedBw) );
             */
             //: Example text: "Syncing 'foo.txt', 'bar.txt'"
-            fileProgressString = tr("Syncing %1").arg(allFilenames.join(QChar()));
+            fileProgressString = tr("Syncing %1").arg(allFilenames.join(QStringLiteral(", ")));
             if (estimatedDownBw > 0) {
                 fileProgressString.append(tr(", "));
 // ifdefs: https://github.com/owncloud/client/issues/3095#issuecomment-128409294
@@ -1081,23 +1144,11 @@ void FolderStatusModel::slotFolderSyncStateChange(Folder *f)
         // Reset progress info.
         pi = SubFolderInfo::Progress();
     } else if (state == SyncResult::NotYetStarted) {
-        FolderMan *folderMan = FolderMan::instance();
-        int pos = folderMan->scheduleQueue().indexOf(f);
-        for (auto other : folderMan->map()) {
-            if (other != f && other->isSyncRunning())
-                pos += 1;
-        }
-        QString message;
-        if (pos <= 0) {
-            message = tr("Waiting...");
-        } else {
-            message = tr("Waiting for %n other folder(s)...", "", pos);
-        }
+        pi._overallSyncString = tr("Queued");
         pi = SubFolderInfo::Progress();
-        pi._overallSyncString = message;
     } else if (state == SyncResult::SyncPrepare) {
         pi = SubFolderInfo::Progress();
-        pi._overallSyncString = tr("Preparing to sync...");
+        pi._overallSyncString = Utility::enumToDisplayName(SyncResult::SyncPrepare);
     } else if (state == SyncResult::Problem || state == SyncResult::Success) {
         // Reset the progress info after a sync.
         pi = SubFolderInfo::Progress();
@@ -1115,14 +1166,6 @@ void FolderStatusModel::slotFolderSyncStateChange(Folder *f)
     }
 }
 
-void FolderStatusModel::slotFolderScheduleQueueChanged()
-{
-    // Update messages on waiting folders.
-    for (auto *f : FolderMan::instance()->map()) {
-        slotFolderSyncStateChange(f);
-    }
-}
-
 void FolderStatusModel::resetFolders()
 {
     setAccountState(_accountState);
@@ -1132,7 +1175,7 @@ void FolderStatusModel::slotSyncAllPendingBigFolders()
 {
     for (int i = 0; i < _folders.count(); ++i) {
         if (!_folders[i]._fetched) {
-            _folders[i]._folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, QStringList());
+            _folders[i]._folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, {});
             continue;
         }
         auto folder = _folders.at(i)._folder;
@@ -1155,9 +1198,7 @@ void FolderStatusModel::slotSyncAllPendingBigFolders()
             qCWarning(lcFolderStatus) << "Could not read selective sync list from db.";
             return;
         }
-        for (const auto &undecidedFolder : undecidedList) {
-            blackList.removeAll(undecidedFolder);
-        }
+        blackList -= undecidedList;
         folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, blackList);
 
         // Add all undecided folders to the white list
@@ -1170,7 +1211,7 @@ void FolderStatusModel::slotSyncAllPendingBigFolders()
         folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncWhiteList, whiteList);
 
         // Clear the undecided list
-        folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, QStringList());
+        folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, {});
 
         // Trigger a sync
         if (folder->isSyncRunning()) {
@@ -1184,7 +1225,7 @@ void FolderStatusModel::slotSyncAllPendingBigFolders()
         }
         // Also make sure we see the local file that had been ignored before
         folder->slotNextSyncFullLocalDiscovery();
-        FolderMan::instance()->scheduleFolder(folder);
+        FolderMan::instance()->scheduler()->enqueueFolder(folder);
     }
 
     resetFolders();
@@ -1196,7 +1237,7 @@ void FolderStatusModel::slotSyncNoPendingBigFolders()
         auto folder = _folders.at(i)._folder;
 
         // clear the undecided list
-        folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, QStringList());
+        folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, {});
     }
 
     resetFolders();
@@ -1246,7 +1287,7 @@ bool FolderStatusModel::SubFolderInfo::hasLabel() const
     return _hasError || _fetchingLabel;
 }
 
-void FolderStatusModel::SubFolderInfo::resetSubs(FolderStatusModel *model, QModelIndex index)
+void FolderStatusModel::SubFolderInfo::resetSubs(FolderStatusModel *model, const QModelIndex &index)
 {
     _fetched = false;
     if (_fetchingJob) {
