@@ -21,6 +21,8 @@
 #include "sharemanager.h"
 #include "guiutility.h"
 
+#include "resources/resources.h"
+
 #include "QProgressIndicator.h"
 #include <QBuffer>
 #include <QClipboard>
@@ -206,8 +208,7 @@ void ShareLinkWidget::getShares()
 
 void ShareLinkWidget::slotSharesFetched(const QList<QSharedPointer<Share>> &shares)
 {
-    const QString versionString = _account->serverVersion();
-    qCInfo(lcSharing) << versionString << "Fetched" << shares.count() << "shares";
+    qCInfo(lcSharing) << "Fetched" << shares.count() << "shares";
 
     // Select the share that was previously selected,
     // except if an explicit override was asked for
@@ -227,8 +228,7 @@ void ShareLinkWidget::slotSharesFetched(const QList<QSharedPointer<Share>> &shar
     table->setRowCount(0);
     connect(table, &QTableWidget::itemSelectionChanged, this, &ShareLinkWidget::slotShareSelectionChanged);
 
-    auto deleteIcon = QIcon::fromTheme(QStringLiteral("user-trash"),
-        Utility::getCoreIcon(QStringLiteral("delete")));
+    auto deleteIcon = QIcon::fromTheme(QStringLiteral("user-trash"), Resources::getCoreIcon(QStringLiteral("delete")));
 
     for (const auto &share : shares) {
         if (share->getShareType() != Share::TypeLink) {
@@ -258,7 +258,7 @@ void ShareLinkWidget::slotSharesFetched(const QList<QSharedPointer<Share>> &shar
         table->setItem(row, 0, nameItem);
 
         auto dotdotdotButton = new QToolButton;
-        dotdotdotButton->setText("...");
+        dotdotdotButton->setText(QStringLiteral("..."));
         dotdotdotButton->setProperty(propertyShareC, QVariant::fromValue(linkShare));
         connect(dotdotdotButton, &QAbstractButton::clicked, this, &ShareLinkWidget::slotContextMenuButtonClicked);
         table->setCellWidget(row, 1, dotdotdotButton);
@@ -368,7 +368,7 @@ void ShareLinkWidget::slotShareSelectionChanged()
         // only if setting was successful, we want to show a placeholder afterwards to signalize success
         if (share && share->isPasswordSet()) {
             _ui->checkBox_password->setChecked(true);
-            _ui->lineEdit_password->setPlaceholderText("********");
+            _ui->lineEdit_password->setPlaceholderText(QStringLiteral("********"));
             _ui->lineEdit_password->setEnabled(true);
         }
         _ui->lineEdit_password->setText(QString());
@@ -482,7 +482,7 @@ void ShareLinkWidget::slotPasswordSet()
     _ui->checkBox_password->setEnabled(true);
     _ui->lineEdit_password->setText(QString());
     if (share->isPasswordSet()) {
-        _ui->lineEdit_password->setPlaceholderText("********");
+        _ui->lineEdit_password->setPlaceholderText(QStringLiteral("********"));
         _ui->lineEdit_password->setEnabled(true);
     } else {
         _ui->lineEdit_password->setPlaceholderText(QString());
@@ -573,7 +573,7 @@ void ShareLinkWidget::slotCheckBoxExpireClicked()
 
 void ShareLinkWidget::emailShareLink(const QUrl &url)
 {
-    QString fileName = _sharePath.mid(_sharePath.lastIndexOf('/') + 1);
+    QString fileName = _sharePath.mid(_sharePath.lastIndexOf(QLatin1Char('/')) + 1);
     Utility::openEmailComposer(
         tr("I shared %1 with you").arg(fileName),
         url.toString(),
