@@ -12,31 +12,28 @@
  * for more details.
  */
 
-#include "theme.h"
-#include "configfile.h"
-#include "common/utility.h"
 #include "accessmanager.h"
 #include "application.h"
+#include "common/utility.h"
+#include "common/version.h"
+#include "configfile.h"
+#include "theme.h"
 
 #include "updater/ocupdater.h"
+#include "updater/updater_private.h"
 
 #include <QObject>
 #include <QtCore>
-#include <QtNetwork>
 #include <QtGui>
+#include <QtNetwork>
 #include <QtWidgets>
 
 using namespace std::chrono_literals;
 
+using namespace std::chrono_literals;
+
 namespace OCC {
 
-static const char updateAvailableC[] = "Updater/updateAvailable";
-static const char updateTargetVersionC[] = "Updater/updateTargetVersion";
-static const char updateTargetVersionStringC[] = "Updater/updateTargetVersionString";
-static const char seenVersionC[] = "Updater/seenVersion";
-static const char autoUpdateAttemptedC[] = "Updater/autoUpdateAttempted";
-
-
 UpdaterScheduler::UpdaterScheduler(QObject *parent)
     : QObject(parent)
 {
@@ -106,7 +103,7 @@ void OCUpdater::setUpdateUrl(const QUrl &url)
 
 bool OCUpdater::performUpdate()
 {
-    QSettings settings(ConfigFile::configFile(), QSettings::IniFormat);
+    auto settings = ConfigFile::makeQSettings();
     QString updateFile = settings.value(updateAvailableC).toString();
     if (!updateFile.isEmpty() && QFile(updateFile).exists()
         && !updateSucceeded() /* Someone might have run the updater manually between restarts */) {
@@ -178,7 +175,7 @@ QString OCUpdater::statusString() const
     }
 }
 
-int OCUpdater::downloadState() const
+OCUpdater::DownloadState OCUpdater::downloadState() const
 {
     return _state;
 }
@@ -199,34 +196,33 @@ void OCUpdater::setDownloadState(DownloadState state)
 
 void OCUpdater::slotStartInstaller()
 {
-    QSettings settings(ConfigFile::configFile(), QSettings::IniFormat);
+    auto settings = ConfigFile::makeQSettings();
     QString updateFile = settings.value(updateAvailableC).toString();
     settings.setValue(autoUpdateAttemptedC, true);
     settings.sync();
     qCInfo(lcUpdater) << "Running updater" << updateFile;
 
-    if(updateFile.endsWith(".exe")) {
-        QProcess::startDetached(updateFile, QStringList() << "/S"
-                                                          << "/launch");
-    } else if(updateFile.endsWith(".msi")) {
+    if (updateFile.endsWith(QLatin1String(".exe"))) {
+        QProcess::startDetached(updateFile, QStringList() << QStringLiteral("/S") << QStringLiteral("/launch"));
+    } else if (updateFile.endsWith(QLatin1String(".msi"))) {
         // When MSIs are installed without gui they cannot launch applications
         // as they lack the user context. That is why we need to run the client
         // manually here. We wrap the msiexec and client invocation in a powershell
         // script because owncloud.exe will be shut down for installation.
         // | Out-Null forces powershell to wait for msiexec to finish.
         auto preparePathForPowershell = [](QString path) {
-            path.replace("'", "''");
+            path.replace(QLatin1String("'"), QLatin1String("''"));
 
             return QDir::toNativeSeparators(path);
         };
 
-        QString msiLogFile = ConfigFile::configPath() + "msi.log";
+        QString msiLogFile = ConfigFile::configPath() + QStringLiteral("msi.log");
         const QString command = QStringLiteral("&{msiexec /norestart /passive /i '%1' /L*V '%2'| Out-Null ; &'%3'}")
                                     .arg(preparePathForPowershell(updateFile),
                                         preparePathForPowershell(msiLogFile),
                                         preparePathForPowershell(QCoreApplication::applicationFilePath()));
 
-        QProcess::startDetached("powershell.exe", QStringList{"-Command", command});
+        QProcess::startDetached(QStringLiteral("powershell.exe"), QStringList { QStringLiteral("-Command"), command });
     }
 }
 
@@ -234,7 +230,7 @@ void OCUpdater::checkForUpdate()
 {
     QNetworkReply *reply = _accessManager->get(QNetworkRequest(_updateUrl));
     connect(_timeoutWatchdog, &QTimer::timeout, this, &OCUpdater::slotTimedOut);
-    _timeoutWatchdog->start(30 * 1000);
+    _timeoutWatchdog->start(30s);
     connect(reply, &QNetworkReply::finished, this, &OCUpdater::slotVersionInfoArrived);
 
     setDownloadState(CheckingServer);
@@ -242,16 +238,15 @@ void OCUpdater::checkForUpdate()
 
 void OCUpdater::slotOpenUpdateUrl()
 {
-    QDesktopServices::openUrl(_updateInfo.web());
+    QDesktopServices::openUrl(QUrl(_updateInfo.web()));
 }
 
 bool OCUpdater::updateSucceeded() const
 {
-    QSettings settings(ConfigFile::configFile(), QSettings::IniFormat);
+    auto settings = ConfigFile::makeQSettings();
 
-    qint64 targetVersionInt = Helper::stringVersionToInt(settings.value(updateTargetVersionC).toString());
-    qint64 currentVersion = Helper::currentVersionToInt();
-    return currentVersion >= targetVersionInt;
+    const auto targetVersionInt = QVersionNumber::fromString(settings.value(updateTargetVersionC).toString());
+    return Version::versionWithBuildNumber() >= targetVersionInt;
 }
 
 void OCUpdater::slotVersionInfoArrived()
@@ -284,6 +279,23 @@ void OCUpdater::slotTimedOut()
     setDownloadState(DownloadTimedOut);
 }
 
+QVersionNumber OCUpdater::previouslySkippedVersion()
+{
+    auto settings = ConfigFile::makeQSettings();
+    return QVersionNumber::fromString(settings.value(previouslySkippedVersionC).toString());
+}
+
+void OCUpdater::setPreviouslySkippedVersion(const QVersionNumber &previouslySkippedVersion)
+{
+    setPreviouslySkippedVersion(previouslySkippedVersion.toString());
+}
+
+void OCUpdater::setPreviouslySkippedVersion(const QString &previouslySkippedVersionString)
+{
+    auto settings = ConfigFile::makeQSettings();
+    settings.setValue(previouslySkippedVersionC, previouslySkippedVersionString);
+}
+
 ////////////////////////////////////////////////////////////////////////
 
 NSISUpdater::NSISUpdater(const QUrl &url)
@@ -301,7 +313,7 @@ void NSISUpdater::slotWriteFile()
 
 void NSISUpdater::wipeUpdateData()
 {
-    QSettings settings(ConfigFile::configFile(), QSettings::IniFormat);
+    auto settings = ConfigFile::makeQSettings();
     QString updateFileName = settings.value(updateAvailableC).toString();
     if (!updateFileName.isEmpty())
         QFile::remove(updateFileName);
@@ -314,16 +326,21 @@ void NSISUpdater::wipeUpdateData()
 void NSISUpdater::slotDownloadFinished()
 {
     QNetworkReply *reply = qobject_cast<QNetworkReply *>(sender());
+    const QUrl url(reply->url());
     reply->deleteLater();
+    _file->close();
     if (reply->error() != QNetworkReply::NoError) {
         setDownloadState(DownloadFailed);
         return;
     }
+    const auto status = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute);
+    if (status != 200) {
+        setDownloadState(DownloadFailed);
+        return;
+    }
 
-    QUrl url(reply->url());
-    _file->close();
 
-    QSettings settings(ConfigFile::configFile(), QSettings::IniFormat);
+    auto settings = ConfigFile::makeQSettings();
 
     // remove previously downloaded but not used installer
     QFile oldTargetFile(settings.value(updateAvailableC).toString());
@@ -341,36 +358,36 @@ void NSISUpdater::slotDownloadFinished()
 
 void NSISUpdater::versionInfoArrived(const UpdateInfo &info)
 {
-    QSettings settings(ConfigFile::configFile(), QSettings::IniFormat);
-    qint64 infoVersion = Helper::stringVersionToInt(info.version());
-    auto seenString = settings.value(seenVersionC).toString();
-    qint64 seenVersion = Helper::stringVersionToInt(seenString);
-    qint64 currVersion = Helper::currentVersionToInt();
+    auto settings = ConfigFile::makeQSettings();
+    const auto infoVersion = QVersionNumber::fromString(info.version());
+    const auto previouslySkippedVersion = this->previouslySkippedVersion();
     qCInfo(lcUpdater) << "Version info arrived:"
-            << "Your version:" << currVersion
-            << "Skipped version:" << seenVersion << seenString
-            << "Available version:" << infoVersion << info.version()
-            << "Available version string:" << info.versionString()
-            << "Web url:" << info.web()
-            << "Download url:" << info.downloadUrl();
+                      << "Your version:" << Version::versionWithBuildNumber()
+                      << "Skipped version:" << previouslySkippedVersion
+                      << "Available version:" << infoVersion << info.version()
+                      << "Available version string:" << info.versionString()
+                      << "Web url:" << info.web()
+                      << "Download url:" << info.downloadUrl();
     if (info.version().isEmpty())
     {
         qCInfo(lcUpdater) << "No version information available at the moment";
         setDownloadState(UpToDate);
-    } else if (infoVersion <= currVersion
-               || infoVersion <= seenVersion) {
+    } else if (infoVersion <= Version::versionWithBuildNumber()
+        || infoVersion <= previouslySkippedVersion) {
         qCInfo(lcUpdater) << "Client is on latest version!";
         setDownloadState(UpToDate);
     } else {
-        QString url = info.downloadUrl();
+        const QString url = info.downloadUrl();
         if (url.isEmpty()) {
             showNoUrlDialog(info);
         } else {
-            _targetFile = ConfigFile::configPath() + url.mid(url.lastIndexOf('/') + 1);
-            if (QFile(_targetFile).exists()) {
+            _targetFile = ConfigFile::configPath() + url.mid(url.lastIndexOf(QLatin1Char('/')) + 1);
+            if (QFile::exists(_targetFile)) {
                 setDownloadState(DownloadComplete);
             } else {
-                QNetworkReply *reply = qnam()->get(QNetworkRequest(QUrl(url)));
+                auto request = QNetworkRequest(QUrl(url));
+                request.setAttribute(QNetworkRequest::RedirectPolicyAttribute, QNetworkRequest::NoLessSafeRedirectPolicy);
+                QNetworkReply *reply = qnam()->get(request);
                 connect(reply, &QIODevice::readyRead, this, &NSISUpdater::slotWriteFile);
                 connect(reply, &QNetworkReply::finished, this, &NSISUpdater::slotDownloadFinished);
                 setDownloadState(Downloading);
@@ -387,7 +404,6 @@ void NSISUpdater::showNoUrlDialog(const UpdateInfo &info)
     // if the version tag is set, there is a newer version.
     QDialog *msgBox = new QDialog;
     msgBox->setAttribute(Qt::WA_DeleteOnClose);
-    msgBox->setWindowFlags(msgBox->windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
     QIcon infoIcon = msgBox->style()->standardIcon(QStyle::SP_MessageBoxInformation);
     int iconSize = msgBox->style()->pixelMetric(QStyle::PM_MessageBoxIconSize);
@@ -407,7 +423,7 @@ void NSISUpdater::showNoUrlDialog(const UpdateInfo &info)
     QString txt = tr("<p>A new version of the %1 Client is available.</p>"
                      "<p><b>%2</b> is available for download. The installed version is %3.</p>")
                       .arg(Utility::escape(Theme::instance()->appNameGUI()),
-                          Utility::escape(info.versionString()), Utility::escape(clientVersion()));
+                          Utility::escape(info.versionString()), Utility::escape(Version::versionWithBuildNumber().toString()));
 
     lbl->setText(txt);
     lbl->setTextFormat(Qt::RichText);
@@ -425,7 +441,7 @@ void NSISUpdater::showNoUrlDialog(const UpdateInfo &info)
     connect(reject, &QAbstractButton::clicked, msgBox, &QDialog::reject);
     connect(getupdate, &QAbstractButton::clicked, msgBox, &QDialog::accept);
 
-    connect(skip, &QAbstractButton::clicked, this, &NSISUpdater::slotSetSeenVersion);
+    connect(skip, &QAbstractButton::clicked, this, &NSISUpdater::slotSetPreviouslySkippedVersion);
     connect(getupdate, &QAbstractButton::clicked, this, &NSISUpdater::slotOpenUpdateUrl);
 
     layout->addWidget(bb);
@@ -438,7 +454,6 @@ void NSISUpdater::showUpdateErrorDialog(const QString &targetVersion)
 {
     QDialog *msgBox = new QDialog;
     msgBox->setAttribute(Qt::WA_DeleteOnClose);
-    msgBox->setWindowFlags(msgBox->windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
     QIcon infoIcon = msgBox->style()->standardIcon(QStyle::SP_MessageBoxInformation);
     int iconSize = msgBox->style()->pixelMetric(QStyle::PM_MessageBoxIconSize);
@@ -458,7 +473,7 @@ void NSISUpdater::showUpdateErrorDialog(const QString &targetVersion)
     QString txt = tr("<p>A new version of the %1 Client is available but the updating process failed.</p>"
                      "<p><b>%2</b> has been downloaded. The installed version is %3.</p>")
                       .arg(Utility::escape(Theme::instance()->appNameGUI()),
-                          Utility::escape(targetVersion), Utility::escape(clientVersion()));
+                          Utility::escape(targetVersion), Utility::escape(Version::versionWithBuildNumber().toString()));
 
     lbl->setText(txt);
     lbl->setTextFormat(Qt::RichText);
@@ -480,7 +495,7 @@ void NSISUpdater::showUpdateErrorDialog(const QString &targetVersion)
 
     connect(skip, &QAbstractButton::clicked, this, [this]() {
         wipeUpdateData();
-        slotSetSeenVersion();
+        slotSetPreviouslySkippedVersion();
     });
     // askagain: do nothing
     connect(retry, &QAbstractButton::clicked, this, [this]() {
@@ -499,7 +514,7 @@ void NSISUpdater::showUpdateErrorDialog(const QString &targetVersion)
 bool NSISUpdater::handleStartup()
 {
     ConfigFile cfg;
-    QSettings settings(ConfigFile::configFile(), QSettings::IniFormat);
+    auto settings = ConfigFile::makeQSettings();
     QString updateFileName = settings.value(updateAvailableC).toString();
     // has the previous run downloaded an update?
     if (!updateFileName.isEmpty() && QFile(updateFileName).exists()) {
@@ -509,7 +524,7 @@ bool NSISUpdater::handleStartup()
             if (updateSucceeded()) {
                 // success: clean up
                 qCInfo(lcUpdater) << "The requested update attempt has succeeded"
-                        << Helper::currentVersionToInt();
+                                  << Version::versionWithBuildNumber();
                 wipeUpdateData();
                 return false;
             } else {
@@ -527,31 +542,36 @@ bool NSISUpdater::handleStartup()
     return false;
 }
 
-void NSISUpdater::slotSetSeenVersion()
+void NSISUpdater::slotSetPreviouslySkippedVersion()
 {
-    QSettings settings(ConfigFile::configFile(), QSettings::IniFormat);
-    settings.setValue(seenVersionC, updateInfo().version());
+    setPreviouslySkippedVersion(updateInfo().version());
 }
 
 ////////////////////////////////////////////////////////////////////////
 
+namespace {
+    QDateTime applicationMTime()
+    {
+        return QFileInfo(QApplication::applicationFilePath()).lastModified().toUTC();
+    }
+}
+
 PassiveUpdateNotifier::PassiveUpdateNotifier(const QUrl &url)
     : OCUpdater(url)
+    , _initialAppMTime(applicationMTime())
 {
-    // remember the version of the currently running binary. On Linux it might happen that the
-    // package management updates the package while the app is running. This is detected in the
-    // updater slot: If the installed binary on the hd has a different version than the one
-    // running, the running app is restarted. That happens in folderman.
-    _runningAppVersion = Utility::versionOfInstalledBinary();
 }
 
 void PassiveUpdateNotifier::backgroundCheckForUpdate()
 {
-    if (Utility::isLinux()) {
-        // on linux, check if the installed binary is still the same version
-        // as the one that is running. If not, restart if possible.
-        const QByteArray fsVersion = Utility::versionOfInstalledBinary();
-        if (!(fsVersion.isEmpty() || _runningAppVersion.isEmpty()) && fsVersion != _runningAppVersion) {
+    if (Utility::isLinux() && !Utility::runningInAppImage()) {
+        // a bit of a naive approach, since it doesn't exclude downgrades or forced reinstalls
+        // whenever the application binary's mtime changes, we assume it has changed, since users are expected not to call touch on it or otherwise change it
+        // manually
+        const auto currentMTime = applicationMTime();
+        qCDebug(lcUpdater) << "initial mtime:" << _initialAppMTime << "current mtime:" << currentMTime;
+        if (currentMTime != _initialAppMTime) {
+            qCInfo(lcUpdater) << "Binary mtime changed since application started, requesting restart";
             emit requestRestart();
         }
     }
@@ -561,10 +581,8 @@ void PassiveUpdateNotifier::backgroundCheckForUpdate()
 
 void PassiveUpdateNotifier::versionInfoArrived(const UpdateInfo &info)
 {
-    qint64 currentVer = Helper::currentVersionToInt();
-    qint64 remoteVer = Helper::stringVersionToInt(info.version());
-
-    if (info.version().isEmpty() || currentVer >= remoteVer) {
+    const auto remoteVer = QVersionNumber::fromString(info.version());
+    if (info.version().isEmpty() || Version::versionWithBuildNumber() >= remoteVer) {
         qCInfo(lcUpdater) << "Client is on latest version!";
         setDownloadState(UpToDate);
     } else {
