@@ -15,70 +15,45 @@
 
 #include "syncengine.h"
 #include "account.h"
-#include "owncloudpropagator.h"
+#include "common/asserts.h"
+#include "common/syncfilestatus.h"
 #include "common/syncjournaldb.h"
 #include "common/syncjournalfilerecord.h"
-#include "discoveryphase.h"
-#include "creds/abstractcredentials.h"
-#include "common/syncfilestatus.h"
-#include "csync_exclude.h"
-#include "filesystem.h"
-#include "propagateremotedelete.h"
-#include "propagatedownload.h"
-#include "common/asserts.h"
-#include "discovery.h"
 #include "common/vfs.h"
+#include "configfile.h"
+#include "creds/abstractcredentials.h"
+#include "csync_exclude.h"
+#include "discovery.h"
+#include "discoveryphase.h"
+#include "filesystem.h"
+#include "owncloudpropagator.h"
+#include "propagatedownload.h"
+#include "propagateremotedelete.h"
 
-#ifdef Q_OS_WIN
-#include <windows.h>
-#else
-#include <unistd.h>
-#endif
-
-#include <climits>
-#include <assert.h>
 #include <chrono>
 
 #include <QCoreApplication>
-#include <QSslSocket>
 #include <QDir>
 #include <QLoggingCategory>
-#include <QMutexLocker>
-#include <QThread>
+#include <QProcess>
+#include <QSslSocket>
 #include <QStringList>
 #include <QTextStream>
 #include <QTime>
 #include <QUrl>
-#include <QSslCertificate>
-#include <QProcess>
-#include <QElapsedTimer>
-#include <qtextcodec.h>
+
+using namespace std::chrono_literals;
 
 namespace OCC {
 
 Q_LOGGING_CATEGORY(lcEngine, "sync.engine", QtInfoMsg)
 
-/** When the client touches a file, block change notifications for this duration (ms)
- *
- * On Linux and Windows the file watcher can't distinguish a change that originates
- * from the client (like a download during a sync operation) and an external change.
- * To work around that, all files the client touches are recorded and file change
- * notifications for these are blocked for some time. This value controls for how
- * long.
- *
- * Reasons this delay can't be very small:
- * - it takes time for the change notification to arrive and to be processed by the client
- * - some time could pass between the client recording that a file will be touched
- *   and its filesystem operation finishing, triggering the notification
- */
-static const std::chrono::milliseconds s_touchedFilesMaxAgeMs(3 * 1000);
-
 // doc in header
-std::chrono::milliseconds SyncEngine::minimumFileAgeForUpload(2000);
+std::chrono::seconds SyncEngine::minimumFileAgeForUpload(2s);
 
-SyncEngine::SyncEngine(AccountPtr account, const QString &localPath,
-    const QString &remotePath, OCC::SyncJournalDb *journal)
+SyncEngine::SyncEngine(AccountPtr account, const QUrl &baseUrl, const QString &localPath, const QString &remotePath, OCC::SyncJournalDb *journal)
     : _account(account)
+    , _baseUrl(baseUrl)
     , _needsUpdate(false)
     , _syncRunning(false)
     , _localPath(localPath)
@@ -89,7 +64,7 @@ SyncEngine::SyncEngine(AccountPtr account, const QString &localPath,
     , _hasRemoveFile(false)
     , _uploadLimit(0)
     , _downloadLimit(0)
-    , _anotherSyncNeeded(NoFollowUpSync)
+    , _anotherSyncNeeded(AnotherSyncNeeded::NoFollowUpSync)
 {
     qRegisterMetaType<SyncFileItem>("SyncFileItem");
     qRegisterMetaType<SyncFileItemPtr>("SyncFileItemPtr");
@@ -104,10 +79,6 @@ SyncEngine::SyncEngine(AccountPtr account, const QString &localPath,
     _excludedFiles.reset(new ExcludedFiles);
 
     _syncFileStatusTracker.reset(new SyncFileStatusTracker(this));
-
-    _clearTouchedFilesTimer.setSingleShot(true);
-    _clearTouchedFilesTimer.setInterval(30 * 1000);
-    connect(&_clearTouchedFilesTimer, &QTimer::timeout, this, &SyncEngine::slotClearTouchedFiles);
 }
 
 SyncEngine::~SyncEngine()
@@ -163,7 +134,7 @@ bool SyncEngine::checkErrorBlacklisting(SyncFileItem &item)
         if (item._etag.isEmpty() || entry._lastTryEtag.isEmpty()) {
             qCInfo(lcEngine) << item._file << "one ETag is empty, no blacklisting";
             return false;
-        } else if (item._etag != entry._lastTryEtag) {
+        } else if (item._etag.toUtf8() != entry._lastTryEtag) {
             qCInfo(lcEngine) << item._file << " is blacklisted, but has changed etag!";
             return false;
         }
@@ -177,7 +148,7 @@ bool SyncEngine::checkErrorBlacklisting(SyncFileItem &item)
     // for reporting and for making sure we don't update the blacklist
     // entry yet.
     // Classification is this _instruction and _status
-    item._instruction = CSYNC_INSTRUCTION_IGNORE;
+    item.setInstruction(CSYNC_INSTRUCTION_IGNORE);
     if (entry._errorCategory == SyncJournalErrorBlacklistRecord::Category::LocalSoftError) {
         item._status = SyncFileItem::SoftError;
         item._errorString = entry._errorString;
@@ -207,9 +178,7 @@ void SyncEngine::deleteStaleDownloadInfos(const SyncFileItemSet &syncItems)
     // Find all downloadinfo paths that we want to preserve.
     QSet<QString> download_file_paths;
     for (const auto &it : syncItems) {
-        if (it->_direction == SyncFileItem::Down
-            && it->_type == ItemTypeFile
-            && isFileTransferInstruction(it->_instruction)) {
+        if (it->_direction == SyncFileItem::Down && it->_type == ItemTypeFile && isFileTransferInstruction(it->instruction())) {
             download_file_paths.insert(it->_file);
         }
     }
@@ -229,9 +198,7 @@ void SyncEngine::deleteStaleUploadInfos(const SyncFileItemSet &syncItems)
     // Find all blacklisted paths that we want to preserve.
     QSet<QString> upload_file_paths;
     for (const auto &it : syncItems) {
-        if (it->_direction == SyncFileItem::Up
-            && it->_type == ItemTypeFile
-            && isFileTransferInstruction(it->_instruction)) {
+        if (it->_direction == SyncFileItem::Up && it->_type == ItemTypeFile && isFileTransferInstruction(it->instruction())) {
             upload_file_paths.insert(it->_file);
         }
     }
@@ -244,8 +211,7 @@ void SyncEngine::deleteStaleUploadInfos(const SyncFileItemSet &syncItems)
         for (auto transferId : ids) {
             if (!transferId)
                 continue; // Was not a chunked upload
-            QUrl url = Utility::concatUrlPath(account()->url(), QLatin1String("remote.php/dav/uploads/") + account()->davUser() + QLatin1Char('/') + QString::number(transferId));
-            (new DeleteJob(account(), url, this))->start();
+            (new DeleteJob(account(), account()->url(), QLatin1String("remote.php/dav/uploads/") + account()->davUser() + QLatin1Char('/') + QString::number(transferId), this))->start();
         }
     }
 }
@@ -307,25 +273,24 @@ void OCC::SyncEngine::slotItemDiscovered(const OCC::SyncFileItemPtr &item)
 {
     if (Utility::isConflictFile(item->_file))
         _seenConflictFiles.insert(item->_file);
-    if (item->_instruction == CSYNC_INSTRUCTION_UPDATE_METADATA && !item->isDirectory()) {
+    if (item->instruction() == CSYNC_INSTRUCTION_UPDATE_METADATA && !item->isDirectory()) {
         _hasNoneFiles = true;
-    } else if (item->_instruction == CSYNC_INSTRUCTION_NONE) {
+    } else if (item->instruction() == CSYNC_INSTRUCTION_NONE) {
         _hasNoneFiles = true;
         if (_account->capabilities().uploadConflictFiles() && Utility::isConflictFile(item->_file)) {
             // For uploaded conflict files, files with no action performed on them should
             // be displayed: but we mustn't overwrite the instruction if something happens
             // to the file!
             item->_errorString = tr("Unresolved conflict.");
-            item->_instruction = CSYNC_INSTRUCTION_IGNORE;
+            item->setInstruction(CSYNC_INSTRUCTION_IGNORE);
             item->_status = SyncFileItem::Conflict;
         }
         return;
-    } else if (item->_instruction == CSYNC_INSTRUCTION_REMOVE && !item->_isSelectiveSync) {
+    } else if (item->instruction() == CSYNC_INSTRUCTION_REMOVE && !item->_isSelectiveSync) {
         _hasRemoveFile = true;
-    } else if (item->_instruction == CSYNC_INSTRUCTION_RENAME) {
+    } else if (item->instruction() == CSYNC_INSTRUCTION_RENAME) {
         _hasNoneFiles = true; // If a file (or every file) has been renamed, it means not al files where deleted
-    } else if (item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE
-        || item->_instruction == CSYNC_INSTRUCTION_SYNC) {
+    } else if (item->instruction() & (CSYNC_INSTRUCTION_TYPE_CHANGE & CSYNC_INSTRUCTION_SYNC)) {
         if (item->_direction == SyncFileItem::Up) {
             // An upload of an existing file means that the file was left unchanged on the server
             // This counts as a NONE for detecting if all the files on the server were changed
@@ -342,7 +307,8 @@ void OCC::SyncEngine::slotItemDiscovered(const OCC::SyncFileItemPtr &item)
         const auto it = _syncItems.find(item);
         if (it != _syncItems.cend()) {
             const auto &item2 = it->get();
-            qCWarning(lcEngine) << "We already have an item for " << item2->_file << ":" << item2->_instruction << item2->_direction << "|" << item->_instruction << item->_direction;
+            qCWarning(lcEngine) << "We already have an item for " << item2->_file << ":" << item2->instruction() << item2->_direction << "|"
+                                << item->instruction() << item->_direction;
             return false;
         }
         return true;
@@ -364,8 +330,7 @@ void SyncEngine::startSync()
     }
 
     _syncRunning = true;
-    _anotherSyncNeeded = NoFollowUpSync;
-    _clearTouchedFilesTimer.stop();
+    _anotherSyncNeeded = AnotherSyncNeeded::NoFollowUpSync;
 
     _hasNoneFiles = false;
     _hasRemoveFile = false;
@@ -374,7 +339,7 @@ void SyncEngine::startSync()
     _progressInfo->reset();
 
     if (!QFileInfo::exists(_localPath)) {
-        _anotherSyncNeeded = DelayedFollowUp;
+        _anotherSyncNeeded = AnotherSyncNeeded::DelayedFollowUp;
         // No _tr, it should only occur in non-mirall
         Q_EMIT syncError(QStringLiteral("Unable to find local sync folder."));
         finalize(false);
@@ -388,7 +353,7 @@ void SyncEngine::startSync()
         if (freeBytes < minFree) {
             qCWarning(lcEngine()) << "Too little space available at" << _localPath << ". Have"
                                   << freeBytes << "bytes and require at least" << minFree << "bytes";
-            _anotherSyncNeeded = DelayedFollowUp;
+            _anotherSyncNeeded = AnotherSyncNeeded::DelayedFollowUp;
             Q_EMIT syncError(tr("Only %1 are available, need at least %2 to start",
                 "Placeholders are postfixed with file sizes using Utility::octetsToString()")
                                  .arg(
@@ -432,7 +397,7 @@ void SyncEngine::startSync()
 
     _lastLocalDiscoveryStyle = _localDiscoveryStyle;
 
-    if (_syncOptions._vfs->mode() == Vfs::WithSuffix && _syncOptions._vfs->fileSuffix().isEmpty()) {
+    if (syncOptions()._vfs->mode() == Vfs::WithSuffix && syncOptions()._vfs->fileSuffix().isEmpty()) {
         Q_EMIT syncError(tr("Using virtual files with suffix, but suffix is not set"));
         finalize(false);
         return;
@@ -453,16 +418,15 @@ void SyncEngine::startSync()
     _stopWatch.start();
 
     qCInfo(lcEngine) << "#### Discovery start ####################################################";
-    qCInfo(lcEngine) << "Server" << account()->serverVersion()
+    qCInfo(lcEngine) << "Server" << account()->capabilities().status().versionString()
                      << (account()->isHttp2Supported() ? "Using HTTP/2" : "");
     _progressInfo->_status = ProgressInfo::Discovery;
     emit transmissionProgress(*_progressInfo);
 
     // TODO: add a constructor to DiscoveryPhase
     // pass a syncEngine object rather than copying everyhting to another object
-    _discoveryPhase.reset(new DiscoveryPhase);
-    _discoveryPhase->_account = _account;
-    _discoveryPhase->_excludes = _excludedFiles.data();
+    _discoveryPhase.reset(new DiscoveryPhase(_account, syncOptions(), _baseUrl));
+    _discoveryPhase->_excludes = _excludedFiles.get();
     _discoveryPhase->_statedb = _journal;
     _discoveryPhase->_localDir = _localPath;
     if (!_discoveryPhase->_localDir.endsWith(QLatin1Char('/')))
@@ -470,7 +434,6 @@ void SyncEngine::startSync()
     _discoveryPhase->_remoteFolder = _remotePath;
     if (!_discoveryPhase->_remoteFolder.endsWith(QLatin1Char('/')))
         _discoveryPhase->_remoteFolder+=QLatin1Char('/');
-    _discoveryPhase->_syncOptions = _syncOptions;
     _discoveryPhase->_shouldDiscoverLocaly = [this](const QString &s) { return shouldDiscoverLocally(s); };
     _discoveryPhase->setSelectiveSyncBlackList(selectiveSyncBlackList);
     _discoveryPhase->setSelectiveSyncWhiteList(_journal->getSelectiveSyncList(SyncJournalDb::SelectiveSyncWhiteList, &ok));
@@ -481,38 +444,25 @@ void SyncEngine::startSync()
         return;
     }
 
-    // Check for invalid character in old server version
-    QString invalidFilenamePattern = _account->capabilities().invalidFilenameRegex();
-    if (invalidFilenamePattern.isNull()
-        && _account->serverVersionInt() < Account::makeServerVersion(8, 1, 0)) {
-        // Server versions older than 8.1 don't support some characters in filenames.
-        // If the capability is not set, default to a pattern that avoids uploading
-        // files with names that contain these.
-        // It's important to respect the capability also for older servers -- the
-        // version check doesn't make sense for custom servers.
-        invalidFilenamePattern = QLatin1String("[\\\\:?*\"<>|]");
+    const QString invalidFilenamePattern = _account->capabilities().invalidFilenameRegex();
+    if (!invalidFilenamePattern.isEmpty()) {
+        _discoveryPhase->_invalidFilenameRx = QRegularExpression(invalidFilenamePattern);
     }
-    if (!invalidFilenamePattern.isEmpty())
-        _discoveryPhase->_invalidFilenameRx = QRegExp(invalidFilenamePattern);
     _discoveryPhase->_serverBlacklistedFiles = _account->capabilities().blacklistedFiles();
     _discoveryPhase->_ignoreHiddenFiles = ignoreHiddenFiles();
 
-    connect(_discoveryPhase.data(), &DiscoveryPhase::itemDiscovered, this, &SyncEngine::slotItemDiscovered);
-    connect(_discoveryPhase.data(), &DiscoveryPhase::newBigFolder, this, &SyncEngine::newBigFolder);
-    connect(_discoveryPhase.data(), &DiscoveryPhase::fatalError, this, [this](const QString &errorString) {
+    connect(_discoveryPhase.get(), &DiscoveryPhase::itemDiscovered, this, &SyncEngine::slotItemDiscovered);
+    connect(_discoveryPhase.get(), &DiscoveryPhase::newBigFolder, this, &SyncEngine::newBigFolder);
+    connect(_discoveryPhase.get(), &DiscoveryPhase::fatalError, this, [this](const QString &errorString) {
         Q_EMIT syncError(errorString);
         finalize(false);
     });
-    connect(_discoveryPhase.data(), &DiscoveryPhase::finished, this, &SyncEngine::slotDiscoveryFinished);
-    connect(_discoveryPhase.data(), &DiscoveryPhase::silentlyExcluded,
-        _syncFileStatusTracker.data(), &SyncFileStatusTracker::slotAddSilentlyExcluded);
-    connect(_discoveryPhase.data(), &DiscoveryPhase::excluded,
-        _syncFileStatusTracker.data(), &SyncFileStatusTracker::slotAddSilentlyExcluded);
-    connect(_discoveryPhase.data(), &DiscoveryPhase::excluded,
-        this, &SyncEngine::excluded);
+    connect(_discoveryPhase.get(), &DiscoveryPhase::finished, this, &SyncEngine::slotDiscoveryFinished);
+    connect(_discoveryPhase.get(), &DiscoveryPhase::silentlyExcluded, _syncFileStatusTracker.data(), &SyncFileStatusTracker::slotAddSilentlyExcluded);
+    connect(_discoveryPhase.get(), &DiscoveryPhase::excluded, _syncFileStatusTracker.data(), &SyncFileStatusTracker::slotAddSilentlyExcluded);
+    connect(_discoveryPhase.get(), &DiscoveryPhase::excluded, this, &SyncEngine::excluded);
 
-    auto discoveryJob = new ProcessDirectoryJob(
-        _discoveryPhase.data(), PinState::AlwaysLocal, _discoveryPhase.data());
+    auto discoveryJob = new ProcessDirectoryJob(_discoveryPhase.get(), PinState::AlwaysLocal, _discoveryPhase.get());
     _discoveryPhase->startJob(discoveryJob);
     connect(discoveryJob, &ProcessDirectoryJob::etag, this, &SyncEngine::slotRootEtagReceived);
 }
@@ -538,7 +488,7 @@ void SyncEngine::slotFolderDiscovered(bool local, const QString &folder)
     emit transmissionProgress(*_progressInfo);
 }
 
-void SyncEngine::slotRootEtagReceived(const QByteArray &e, const QDateTime &time)
+void SyncEngine::slotRootEtagReceived(const QString &e, const QDateTime &time)
 {
     if (_remoteRootEtag.isEmpty()) {
         qCDebug(lcEngine) << "Root etag:" << e;
@@ -590,21 +540,21 @@ void SyncEngine::slotDiscoveryFinished()
             restoreOldFiles(_syncItems);
         }
 
-        if (_discoveryPhase->_anotherSyncNeeded && _anotherSyncNeeded == NoFollowUpSync) {
-            _anotherSyncNeeded = ImmediateFollowUp;
+        if (_discoveryPhase->_anotherSyncNeeded && _anotherSyncNeeded == AnotherSyncNeeded::NoFollowUpSync) {
+            _anotherSyncNeeded = AnotherSyncNeeded::ImmediateFollowUp;
         }
 
         Q_ASSERT(std::is_sorted(_syncItems.begin(), _syncItems.end()));
 
         const auto regex = syncOptions().fileRegex();
         if (regex.isValid()) {
-            QSet<QStringRef> names;
+            QSet<QStringView> names;
             for (auto &i : _syncItems) {
                 if (regex.match(i->_file).hasMatch()) {
                     int index = -1;
-                    QStringRef ref;
+                    QStringView ref;
                     do {
-                        ref = i->_file.midRef(0, index);
+                        ref = QStringView(i->_file).mid(0, index);
                         names.insert(ref);
                         index = ref.lastIndexOf(QLatin1Char('/'));
                     } while (index > 0);
@@ -623,9 +573,7 @@ void SyncEngine::slotDiscoveryFinished()
                 }
                 return old_size - c.size();
             };
-            erase_if(_syncItems, [&names](const SyncFileItemPtr &i) {
-                return !names.contains(QStringRef { &i->_file });
-            });
+            erase_if(_syncItems, [&names](const SyncFileItemPtr &i) { return !names.contains(QStringView{i->_file}); });
         }
 
         qCInfo(lcEngine) << "#### Reconcile (aboutToPropagate) #################################################### " << _stopWatch.addLapTime(QStringLiteral("Reconcile (aboutToPropagate)")) << "ms";
@@ -644,22 +592,20 @@ void SyncEngine::slotDiscoveryFinished()
 
         // post update phase script: allow to tweak stuff by a custom script in debug mode.
         if (!qEnvironmentVariableIsEmpty("OWNCLOUD_POST_UPDATE_SCRIPT")) {
-    #ifndef NDEBUG
+#ifndef NDEBUG
             const QString script = qEnvironmentVariable("OWNCLOUD_POST_UPDATE_SCRIPT");
 
             qCDebug(lcEngine) << "Post Update Script: " << script;
-            QProcess::execute(script);
-    #else
+            QProcess::execute(script, {});
+#else
             qCWarning(lcEngine) << "**** Attention: POST_UPDATE_SCRIPT installed, but not executed because compiled with NDEBUG";
-    #endif
+#endif
         }
 
         // do a database commit
         _journal->commit(QStringLiteral("post treewalk"));
 
-        _propagator = QSharedPointer<OwncloudPropagator>(
-            new OwncloudPropagator(_account, _localPath, _remotePath, _journal));
-        _propagator->setSyncOptions(_syncOptions);
+        _propagator = QSharedPointer<OwncloudPropagator>::create(_account, syncOptions(), _baseUrl, _localPath, _remotePath, _journal);
         connect(_propagator.data(), &OwncloudPropagator::itemCompleted,
             this, &SyncEngine::slotItemCompleted);
         connect(_propagator.data(), &OwncloudPropagator::progress,
@@ -668,7 +614,6 @@ void SyncEngine::slotDiscoveryFinished()
             this, &SyncEngine::updateFileTotal);
         connect(_propagator.data(), &OwncloudPropagator::finished, this, &SyncEngine::slotPropagationFinished, Qt::QueuedConnection);
         connect(_propagator.data(), &OwncloudPropagator::seenLockedFile, this, &SyncEngine::seenLockedFile);
-        connect(_propagator.data(), &OwncloudPropagator::touchedFile, this, &SyncEngine::slotAddTouchedFile);
         connect(_propagator.data(), &OwncloudPropagator::insufficientLocalStorage, this, &SyncEngine::slotInsufficientLocalStorage);
         connect(_propagator.data(), &OwncloudPropagator::insufficientRemoteStorage, this, &SyncEngine::slotInsufficientRemoteStorage);
         connect(_propagator.data(), &OwncloudPropagator::newItem, this, &SyncEngine::slotNewItem);
@@ -692,32 +637,18 @@ void SyncEngine::slotDiscoveryFinished()
 
     if (!_hasNoneFiles && _hasRemoveFile) {
         qCInfo(lcEngine) << "All the files are going to be changed, asking the user";
-        int side = 0; // > 0 means more deleted on the server.  < 0 means more deleted on the client
-        for (const auto &it : qAsConst(_syncItems)) {
-            if (it->_instruction == CSYNC_INSTRUCTION_REMOVE) {
-                side += it->_direction == SyncFileItem::Down ? 1 : -1;
-            }
-        }
 
-        QPointer<QObject> guard = new QObject();
-        QPointer<QObject> self = this;
-        auto callback = [this, self, finish, guard](bool cancel) -> void {
-            // use a guard to ensure its only called once...
-            // qpointer to self to ensure we still exist
-            if (!guard || !self) {
-                return;
+        if (_promptRemoveAllFiles) {
+            int side = 0; // > 0 means more deleted on the server.  < 0 means more deleted on the client
+            for (const auto &it : qAsConst(_syncItems)) {
+                if (it->instruction() == CSYNC_INSTRUCTION_REMOVE) {
+                    side += it->_direction == SyncFileItem::Down ? 1 : -1;
+                }
             }
-            guard->deleteLater();
-            if (cancel) {
-                qCInfo(lcEngine) << "User aborted sync";
-                finalize(false);
-                return;
-            } else {
-                finish();
-            }
-        };
-        emit aboutToRemoveAllFiles(side >= 0 ? SyncFileItem::Down : SyncFileItem::Up, callback);
-        return;
+            emit aboutToRemoveAllFiles(side >= 0 ? SyncFileItem::Down : SyncFileItem::Up);
+            finalize(false);
+            return;
+        }
     }
     finish();
 }
@@ -727,14 +658,18 @@ void SyncEngine::setNetworkLimits(int upload, int download)
     _uploadLimit = upload;
     _downloadLimit = download;
 
-    if (!_propagator)
-        return;
-
-    _propagator->_uploadLimit = upload;
-    _propagator->_downloadLimit = download;
-
-    if (upload != 0 || download != 0) {
-        qCInfo(lcEngine) << "Network Limits (down/up) " << upload << download;
+    if (_propagator) {
+        if (upload != 0 || download != 0) {
+            qCInfo(lcEngine) << "Network Limits (down/up) " << upload << download;
+            if (!_propagator->_bandwidthManager) {
+                _propagator->_bandwidthManager = new BandwidthManager(_propagator.data());
+            }
+        }
+        // this might set the limit to 0 but only the next sync will have no bandwithd manager set
+        if (_propagator->_bandwidthManager) {
+            _propagator->_bandwidthManager->setCurrentDownloadLimit(download);
+            _propagator->_bandwidthManager->setCurrentUploadLimit(upload);
+        }
     }
 }
 
@@ -755,8 +690,8 @@ void SyncEngine::slotItemCompleted(const SyncFileItemPtr &item)
 
 void SyncEngine::slotPropagationFinished(bool success)
 {
-    if (_propagator->_anotherSyncNeeded && _anotherSyncNeeded == NoFollowUpSync) {
-        _anotherSyncNeeded = ImmediateFollowUp;
+    if (_propagator->_anotherSyncNeeded && _anotherSyncNeeded == AnotherSyncNeeded::NoFollowUpSync) {
+        _anotherSyncNeeded = AnotherSyncNeeded::ImmediateFollowUp;
     }
 
     if (success && _discoveryPhase) {
@@ -765,6 +700,12 @@ void SyncEngine::slotPropagationFinished(bool success)
 
     conflictRecordMaintenance();
 
+    // update placeholders for files that where marked as dirty in a previous run
+    qCInfo(lcEngine) << "Updating files marked as dirty";
+    for (const auto &record : _journal->getFileRecordsWithDirtyPlaceholders()) {
+        _propagator->updateMetadata(*SyncFileItem::fromSyncJournalFileRecord(record));
+    }
+
     _journal->deleteStaleFlagsEntries();
     _journal->commit(QStringLiteral("All Finished."), false);
 
@@ -784,7 +725,7 @@ void SyncEngine::finalize(bool success)
     _stopWatch.stop();
 
     if (_discoveryPhase) {
-        _discoveryPhase.take()->deleteLater();
+        _discoveryPhase.release()->deleteLater();
     }
     _syncRunning = false;
     emit finished(success);
@@ -795,8 +736,6 @@ void SyncEngine::finalize(bool success)
     _uniqueErrors.clear();
     _localDiscoveryPaths.clear();
     _localDiscoveryStyle = LocalDiscoveryStyle::FilesystemOnly;
-
-    _clearTouchedFilesTimer.start();
 }
 
 void SyncEngine::slotProgress(const SyncFileItem &item, qint64 current)
@@ -822,14 +761,14 @@ void SyncEngine::restoreOldFiles(SyncFileItemSet &syncItems)
         if ((*it)->_direction != SyncFileItem::Down)
             continue;
 
-        switch ((*it)->_instruction) {
+        switch ((*it)->instruction()) {
         case CSYNC_INSTRUCTION_SYNC:
             qCWarning(lcEngine) << "restoreOldFiles: RESTORING" << (*it)->_file;
-            (*it)->_instruction = CSYNC_INSTRUCTION_CONFLICT;
+            (*it)->setInstruction(CSYNC_INSTRUCTION_CONFLICT);
             break;
         case CSYNC_INSTRUCTION_REMOVE:
             qCWarning(lcEngine) << "restoreOldFiles: RESTORING" << (*it)->_file;
-            (*it)->_instruction = CSYNC_INSTRUCTION_NEW;
+            (*it)->setInstruction(CSYNC_INSTRUCTION_NEW);
             (*it)->_direction = SyncFileItem::Up;
             break;
         case CSYNC_INSTRUCTION_RENAME:
@@ -843,49 +782,6 @@ void SyncEngine::restoreOldFiles(SyncFileItemSet &syncItems)
     }
 }
 
-void SyncEngine::slotAddTouchedFile(const QString &fn)
-{
-    QElapsedTimer now;
-    now.start();
-    const QString file = QDir::cleanPath(fn);
-
-    // Iterate from the oldest and remove anything older than 15 seconds.
-    while (true) {
-        // don't use a loop as we do erase in here
-        // don't use remove_if as we can stop as soon as we find the first new item
-        auto start = _touchedFiles.cbegin();
-        if (start == _touchedFiles.cend()) {
-            break;
-        }
-        // Compare to our new QElapsedTimer instead of using elapsed().
-        // This avoids querying the current time from the OS for every loop.
-        const auto elapsed = std::chrono::milliseconds(now.msecsSinceReference() - start->first.msecsSinceReference());
-        if (elapsed <= s_touchedFilesMaxAgeMs) {
-            // We found the first path younger than the maximum age, keep the rest.
-            break;
-        }
-        _touchedFiles.erase(start);
-    }
-
-    // This should be the largest QElapsedTimer yet, use constEnd() as hint.
-    _touchedFiles.insert(_touchedFiles.cend(), { now, file });
-}
-
-void SyncEngine::slotClearTouchedFiles()
-{
-    _touchedFiles.clear();
-}
-
-bool SyncEngine::wasFileTouched(const QString &fn) const
-{
-    // Start from the end (most recent) and look for our path. Check the time just in case.
-    for (auto it = _touchedFiles.crbegin(); it != _touchedFiles.crend(); ++it) {
-        if (it->second == fn)
-            return std::chrono::milliseconds(it->first.elapsed()) <= s_touchedFilesMaxAgeMs;
-    }
-    return false;
-}
-
 AccountPtr SyncEngine::account() const
 {
     return _account;
@@ -896,6 +792,13 @@ void SyncEngine::setLocalDiscoveryOptions(LocalDiscoveryStyle style, std::set<QS
     _localDiscoveryStyle = style;
     _localDiscoveryPaths = std::move(paths);
 
+
+    // add running upload to the local discovery
+    const auto info = _journal->getUploadInfos();
+    for (const auto &i : info) {
+        _localDiscoveryPaths.insert(i._path);
+    }
+
     // Normalize to make sure that no path is a contained in another.
     // Note: for simplicity, this code consider anything less than '/' as a path separator, so for
     // example, this will remove "foo.bar" if "foo" is in the list. This will mean we might have
@@ -915,8 +818,9 @@ void SyncEngine::setLocalDiscoveryOptions(LocalDiscoveryStyle style, std::set<QS
 
 bool SyncEngine::shouldDiscoverLocally(const QString &path) const
 {
-    if (_localDiscoveryStyle == LocalDiscoveryStyle::FilesystemOnly)
+    if (_localDiscoveryStyle == LocalDiscoveryStyle::FilesystemOnly) {
         return true;
+    }
 
     // The intention is that if "A/X" is in _localDiscoveryPaths:
     // - parent folders like "/", "A" will be discovered (to make sure the discovery reaches the
@@ -951,31 +855,6 @@ bool SyncEngine::shouldDiscoverLocally(const QString &path) const
     return false;
 }
 
-void SyncEngine::wipeVirtualFiles(const QString &localPath, SyncJournalDb &journal, Vfs &vfs)
-{
-    qCInfo(lcEngine) << "Wiping virtual files inside" << localPath;
-    journal.getFilesBelowPath(QByteArray(), [&](const SyncJournalFileRecord &rec) {
-        if (rec._type != ItemTypeVirtualFile && rec._type != ItemTypeVirtualFileDownload)
-            return;
-
-        qCDebug(lcEngine) << "Removing db record for" << rec._path;
-        journal.deleteFileRecord(QString::fromUtf8(rec._path));
-
-        // If the local file is a dehydrated placeholder, wipe it too.
-        // Otherwise leave it to allow the next sync to have a new-new conflict.
-        QString localFile = localPath + QString::fromUtf8(rec._path);
-        if (QFile::exists(localFile) && vfs.isDehydratedPlaceholder(localFile)) {
-            qCDebug(lcEngine) << "Removing local dehydrated placeholder" << rec._path;
-            FileSystem::remove(localFile);
-        }
-    });
-
-    journal.forceRemoteDiscoveryNextSync();
-
-    // Postcondition: No ItemTypeVirtualFile / ItemTypeVirtualFileDownload left in the db.
-    // But hydrated placeholders may still be around.
-}
-
 void SyncEngine::abort()
 {
     if (_propagator)
@@ -987,8 +866,8 @@ void SyncEngine::abort()
     } else if (_discoveryPhase) {
         // Delete the discovery and all child jobs after ensuring
         // it can't finish and start the propagator
-        disconnect(_discoveryPhase.data(), nullptr, this, nullptr);
-        _discoveryPhase.take()->deleteLater();
+        disconnect(_discoveryPhase.get(), nullptr, this, nullptr);
+        _discoveryPhase.release()->deleteLater();
 
         if (!_goingDown) {
             Q_EMIT syncError(tr("Aborted"));
@@ -1024,4 +903,46 @@ void SyncEngine::slotInsufficientRemoteStorage()
     emit syncError(msg, ErrorCategory::InsufficientRemoteStorage);
 }
 
+bool SyncEngine::isPromtRemoveAllFiles() const
+{
+    return _promptRemoveAllFiles;
+}
+
+void SyncEngine::setPromtRemoveAllFiles(bool promtRemoveAllFiles)
+{
+    _promptRemoveAllFiles = promtRemoveAllFiles;
+}
+
+bool SyncEngine::isExcluded(QStringView filePath) const
+{
+    Q_ASSERT(QDir::isAbsolutePath(filePath.toString()));
+    return _excludedFiles->isExcluded(filePath, localPath(), ignoreHiddenFiles());
+}
+
+bool SyncEngine::loadDefaultExcludes()
+{
+    ConfigFile::setupDefaultExcludeFilePaths(*_excludedFiles);
+    return _excludedFiles->reloadExcludeFiles();
+}
+
+void SyncEngine::clearManualExcludes()
+{
+    _excludedFiles->clearManualExcludes();
+}
+
+bool SyncEngine::reloadExcludes()
+{
+    return _excludedFiles->reloadExcludeFiles();
+}
+
+void SyncEngine::addExcludeList(const QString &filePath)
+{
+    _excludedFiles->addExcludeFilePath(filePath);
+}
+
+void SyncEngine::addManualExclude(const QString &filePath)
+{
+    _excludedFiles->addManualExclude(filePath);
+}
+
 } // namespace OCC
